(* Case based least square SEM - experimental research implementation  
   (c) 2022 Reinhard Oldenburg reinhard.oldenburg@math.uni-augsburg.de *)
   
MyRound[x_, dx_] :=x/;MemberQ[{False,True,E,Pi},x];
MyRound[x_, dx_] := Round[x, dx] /; NumberQ[x];
MyRound[x_, dx_] := x /; Head[x] == Symbol;
MyRound[a_ == b_, dx_] := MyRound[a, dx] == MyRound[b, dx]; 
MyRound[a_ + b_, dx_] := MyRound[a, dx] + MyRound[b, dx]; 
MyRound[a_*b_, dx_] := MyRound[a, dx]*MyRound[b, dx]; 
MyRound[a_^b_, dx_] := MyRound[a, dx]^MyRound[b, dx]; 
MyRound[a_ -> b_, dx_] := Apply[Rule, {a, MyRound[b, dx]}]; 
MyRound[L_List, dx_] := Map[MyRound[#, dx] &, L]; 
MyRound[EVM[i_,j_],_]:= EVM[i,j];
MyRound[s_String,_]:=s;

RoundRule[a_, dx_] := Round[a, dx] /; NumberQ[a]; 
RoundRule[a_ + b_, dx_] := RoundRule[a, dx] + RoundRule[b, dx]; 
RoundRule[a_*b_, dx_] := RoundRule[a, dx]*RoundRule[b, dx]; 
RoundRule[a_^b_, dx_] := RoundRule[a, dx]^RoundRule[b, dx]; 
RoundRule[a_ == b_, dx_] := RoundRule[a, dx] == RoundRule[b, dx]; 
RoundRule[a_ -> b_, dx_] := Apply[Rule, {a, RoundRule[b, dx]}]; 
RoundRule[L_List, dx_] := Map[RoundRule[#, dx] &, L];
RoundRule[a_, dx_] := a /; AtomQ[a] && ! NumberQ[a]; 

SelN[L_]:=Select[L,NumberQ];

   Rhs[(a_) == (b_)] := b; Lhs[(a_) == (b_)] := a;
   headlist = {Or, And, Equal, Unequal, Less, LessEqual, Greater,  GreaterEqual, Inequality};
   getAllVariables[(f_)?NumericQ] := {};
   getAllVariables[{}] := {};
   getAllVariables[a_ -> b_] := 
    Union[getAllVariables[a], getAllVariables[b]];
   getAllVariables[t_] /; MemberQ[headlist, t] := {};
   getAllVariables[ll_List] := 
    Union[Flatten[Union[(getAllVariables[#1] &) /@ ll]]];
   getAllVariables[Derivative[n_Integer][f_][arg__]] := 
    getAllVariables[{arg}];
   getAllVariables[(f_Symbol)[arg__]] := 
    Union[{Module[{fvars}, 
       If[MemberQ[Attributes[f], NumericFunction] || 
         MemberQ[headlist, f], fvars = getAllVariables[{arg}], 
        fvars = f[arg]]; fvars]}]; 
   getAllVariables[other_] := {other};
   GetAllVariables[a_] := Union[Flatten[getAllVariables[a]]];

(**************** SIA ****************)
nnot[a_List] := Map[If[# === Null, Null, 1 - #] &, a];
distri[q_] := 1 - CDF[NormalDistribution[0, 1], q];
normalized[v_List] := 
  Module[{vv = Select[v, NumberQ], mi, ma}, mi = Min[vv]; 
   ma = Max[vv]; (v - mi)/(ma - mi)];
isNormalized[v_List] := 
  Module[{vv = Select[v, NumberQ], mi, ma}, mi = Min[vv]; 
   ma = Max[vv]; 0 <= mi && ma <= 1];
SIA[a_, b_] := 
  Module[{ sa, sb, n, na = 0, naq = 0, nb = 0, nbq = 0, nabq = 0, 
    naqbq = 0, NN = 0, i, j,
    phibino, phi, phinorm, lambda, p, q, Q, k, tau, H1, H2, ent, Phi}, 
   Off@General::munfl;
   If[Length[a] > Length[b], Print["a too long, dropping data"]; 
    a = Take[a, Length@b]]; 
   If[Length[a] < Length[b], Print["b too long, dropping data"]; 
    b = Take[b, Length@a]]; 
   n = Length@a;
   Do[If[NumberQ[a[[i]]] && NumberQ[b[[i]]],
     NN = NN + 1; 
     na = na + a[[i]]; naq = naq + a[[i]]; nb = nb + b[[i]]; 
     nbq = nbq + (1 - b[[i]]); nabq = nabq + a[[i]]*(1 - b[[i]]); 
     naqbq = naqbq + (1 - a[[i]])*(1 - b[[i]]) ],
    {i, 1, n}];
   phibino = 0.0; p = na*nbq/(n*n); Q = nabq; Q = Floor[Q] + 1;
   Do[phibino = phibino + N[n!/k!/(n - k)!*p^k*(1 - p)^(n - k)], {k, 
     Q, n}];
   sa = StandardDeviation@Select[a, NumberQ]; 
   sb = StandardDeviation@Select[b, NumberQ]; 
   If[na*(n - nb) == 0, q = Null,
    q = N[(nabq - na*(n - nb)/n)*
       n^(3/2)/Sqrt[n^2*sa^2 + na^2]/Sqrt[n^2*sb^2 + (n - nb)^2]];
    lambda = N[na*nbq/n] ; phinorm = distri[q]];
   phi = If[lambda > 4, phinorm, phibino];
   ent[x_] := If[x>0,x*Log[2, x],-Infinity];
   H1 = N[-ent[1 - nabq/na] - ent[nabq/na]]; 
   H2 = N[-ent[1 - nabq/nbq] - ent[nabq/nbq]];
   tau = ((1 - H1^2 )*(1 - H2^2));
   If[tau>0,tau=tau^0.25,tau=0];
   If[tau*phi>=0,Phi = Sqrt[tau*phi],Phi =0]; (* entropisch *)
   On@General::munfl;
   If[a==b,Phi=1;phi=1];
   <|"q" -> q, "n" -> n, "na" -> na, "nb" -> nb, "nabq" -> nabq, 
    "lambda" -> lambda, "phinorm" -> phinorm, "phibino" -> phibino, 
    "tau" -> tau,
    "phi" -> phi, "Phi" -> Phi, "H1" -> H1, "H2" -> H2|>
   ];
   (************ End sia ------------------------*)

analyze[res_] := Module[{i, j, m = Length[res["eqs"]],w2},
  Print[{"sol...",Take[res["parasol"],Min[5,Length[res["parasol"]]]]}];
  Print[{"weights", res["weights"], "weights*EVV", 
    res["weights"]*Map[Variance, res["EEs"]]}];
  Print[{"EEs,Mean,SD", Map[Mean, res["EEs"]], 
    Map[StandardDeviation, res["EEs"]]}];
    w2=1/Map[Variance, res["EEs"]]; w2=w2/Apply[Plus,w2];
    Print[{"a-posteriori weight suggestions",w2}];
  Print[{"EEcor", 
    TableForm[
     Round[
      Table[
       Correlation[res["EEs"][[i]] , res["EEs"][[j]] ], {i, 1, m}, {j,
         1, m}], 0.01]]}];
  Print[{"LLs,Mean,SD", Map[Mean, res["LLs"]], 
    Map[StandardDeviation, res["LLs"]]}];
  Print[{"LEcor", 
    TableForm[
     Round[
      Table[
       Correlation[res["LLs"][[i]] , res["EEs"][[j]] ], {i, 1, 
        Length[res["latentvars"]]}, {j, 1, m}], 0.01]]}];
  ];

GOFMC[res_] := 
  Module[{ee = Abs@Flatten@res@"EEcors", 
    de = Abs@Flatten@res@"DEcors", le = Abs@Flatten@res@"LEcors"},
   ee = Mean@Take[Sort@ee, -Ceiling[Length[ee]/3]];
   de = Mean@Take[Sort@de, -Ceiling[Length[de]/3]];
   le = Mean@Take[Sort@le, -Ceiling[Length[le]/3]];
   {(ee + de + le)/3, ee, de, le}
   ];

CalcDataFit[res_]:=CalcDataFit[res,False,-1];
CalcDataFit[res_,weightedByData_]:=CalcDataFit[res,weightedByData,-1];
CalcDataFit[res_,weightedByData_,useWeights_]:=  Module[{i,j,k,m,n=res["n"],data=res["data"],observed=res["observed"],latentvars=res["latentvars"],eqs=res["eqs"],
df,vs,FDFi,soli,iniv,FDF,prog,dsd,posof,ww,StandardDeviation0},
    dsd=Map[StandardDeviation0,Transpose@data]; 
  StandardDeviation0[L_] := StandardDeviation[Select[L, FreeQ[#, Null] &]];
    m=Length@eqs;dsd=Map[StandardDeviation0,Transpose@data];
    ww={}; 
    Do[If[Head[eqs[[j,1]]]==Equal,
        AppendTo[ww,If[useWeights==-1,
                       1,
                       If[useWeights==0,res["weightsPost"][[j]],(res["weightsN"][[j]]+res["weightsPostN"][[j]])/2] ,1] ]],
      {j,1,m}]; 
    FDF=Apply[Plus,ww*Map[(#[[1]]-#[[2]])^2&,Select[Map[First,eqs],(Head[#]==Equal)&]]/.res["sol"]];
            Do[posof[observed[[j]]]=j,{j,1,Length@observed}];
            vs=observed;
            iniv[i_,v_]:= If[NumberQ[data[[i,posof[v]]]],data[[i,posof[v]]],0];
    prog[i_]:= (FDFi=FDF; Do[FDFi=FDFi/. latentvars[[j]]->res["LLs"][[j,i]],{j,1,Length@latentvars}]; 
               FindMinimum[FDFi,Map[{#,iniv[i,#]}&,vs]][[2]] );
    Sqrt[1/(n*Length@ww)*Sum[soli=prog[i]; 
            Apply[Plus,Select[Table[(data[[i,posof[First@soli[[k]]]]]-Last@soli[[k]])^2/
                   If[weightedByData,dsd[[posof[First@soli[[k]]]]]^2,1],{k,1,Length[soli]}],FreeQ[#,Null]&]]/Length[soli] ,
            {i,1,n}]]] ;

 

calcNormedParas[res_] := 
  Module[{NP, sdA, sdB, getSd, subs, t, genRules, getRules1, a, b, 
    eqs = res["eqs"], observed = res["observed"], Variance0, patterns},(*Experimentell-kann auch mit termen arbeiten*)
   Variance0[L_] := Variance[Select[L, FreeQ[#, Null] &]];
   getRules1[{eq_, err_}, p_] := 
    If[MatchQ[eq, l_ == p*r_ + u_],
      {p -> {D[eq[[2]], p], eq[[1]]}},
       If[MatchQ[eq, l_ == p*r_ ],
      {p -> {D[eq[[2]], p], eq[[1]]}},
       {}]];
   getRules1[p_] := Apply[Join, Map[getRules1[#, p] &, eqs]];
   genRules = 
    Apply[Join, Map[getRules1[First[#]] &, res["parasol"]]];
   genRules = 
    Map[(#[[1]] -> (#[[2]] /. 
          Map[(#[[1]] -> 0) &, res["parasol"]])) &, genRules];
   genRules = Select[genRules, FreeQ[#, 0] &];
   subs = 
    Join[Table[
      Apply[Rule, {res["latentvars"][[t]], res["LLs"][[t]]}], {t, 1, 
       Length@res@"latentvars"}], 
     Table[Apply[Rule, {observed[[t]], Transpose[res["data"]][[t]]}], {t, 1, Length@observed}]];
   getSd[a_] := Sqrt@Variance0[a /. subs];
   NP[p_ -> {A_, B_}] := 
    Apply[Rule, {p, (p /. res["sol"])*getSd[A]/getSd[B]}];
   Map[NP, genRules]];

ClearAll[unify]; SetAttributes[$, HoldAll];
Options[unify] = {bindings -> {}, Verbose -> False};
unify[x_, y_, opt___] := 
  Block[{$bindings = bindings /. {opt} /. Options@unify}, 
   Module[{unify0, boundQ, lookup, aslist, i}, 
    aslist[a_] /; Not[AtomQ@a] := 
     Reverse@Table[a[[i]], {i, 1, Length@a}];
    SetAttributes[unify0, Orderless];
    boundQ[x1_] := Module[{}, (x1 /. $bindings) =!= x1];
    lookup[x1_] := Module[{}, x1 /. $bindings];
    unify0[a_, a_] := True;
    unify0[$[x1_], $[y1_]] /; boundQ[$[x1]] && boundQ[$[y1]] := 
     Module[{}, lookup[x1] === lookup[y1]];
    unify0[$[x1_], $[y1_]] /; boundQ[$[x1]] :=  Module[{xval}, xval = lookup[x1];
      AppendTo[$bindings, y1 -> xval]; unify0[xval, $[y1]]];
    unify0[$[x1_], $[y1_]] := Module[{}, AppendTo[$bindings, x1 -> y1]; True];
    unify0[$[x1_], y1_] /; boundQ[x1] := Module[{}, unify0[lookup[x1], y1]];
    unify0[$[x1_], y1_] := Module[{}, AppendTo[$bindings, x1 -> y1]; True];
    unify0[x1_?AtomQ, y1_?AtomQ] := Module[{}, x1 === y1];
    unify0[x1_, y1_] /; 
      Head[x1] == Head[y1] && 
       MemberQ[Apply[Attributes, {Head[x1]}], Orderless] && 
       Length[x1] == Length[y1] := 
     Module[{u, x1l = aslist@x1, y1l = aslist@y1, y1p, res = False, 
       bind = $bindings}, 
      Do[$bindings = bind; 
       If[And @@ Thread[u[x1l, y1p], List] /. u -> unify0, res = True;
         Break[]], {y1p, Permutations@y1l}]; res];
    unify0[x1_, y1_] /; 
      Head[x1] == Head[y1] && Length[x1] == Length[y1] := 
     Module[{u}, And @@ Thread[u[x1, y1], Head[x1]] /. u -> unify0];
    unify0[x1_, y1_] := False;
    If[unify0[x, y], {True, $bindings /. $[a_] :> a}, {False, {}}]]];



Options[flexSEM] = {useDataFit->False, GOFMethod->"GOF1",Opts->{}, OptsF->{}, OptsS->{},includeFast->True, calcDataFitWeighted -> False}; 
flexSEM[data_List, observed_List, latentvars_List, eqs_List,   OptionsPattern[]] := 
  Module[{res,os,of,opt=OptionValue@Opts,optS=OptionValue@OptsS,optF=OptionValue@OptsF,meanGOF },
  of = Join[opt, optF]; os = Join[opt, optS]; 
   res = Join[If[OptionValue@includeFast,{ (* W1 and Ww(Wn) *)
      fastSEM[data,observed,latentvars,eqs,Title->"F_W1",calcDataFit->OptionValue@useDataFit,
           calcDataFitWeighted ->OptionValue@calcDataFitWeighted,Quiet->OptionValue@Quiet],
      fastSEM[data,observed,latentvars,eqs, weightsByLatentCount -> True, Quiet->OptionValue@Quiet,
              Title->"F_Wn",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted],
      fastSEM[data,observed,latentvars,eqs,weightsFromFirstRound -> True, weightsByLatentCountRound0 -> True,
              Quiet->OptionValue@Quiet,
              Title->"F_Ww(Wn)",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted] },{}],
      {Apply[slowSEM,{data,observed,latentvars,eqs}~Join~os~Join~{LastStep->4,onlyFirstStep->False,VarCutFactor->3,InitPV->
      1,useNewWF->0.1,Title->"S1",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],

     Apply[slowSEM,{data,observed,latentvars,eqs}~Join~os~Join~{onlyInitStep -> True, LastStep -> 0, onlyFirstStep -> True,
        InitValsFromSimplifiedModel -> True,Title->"F^a(1)",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],

     Apply[slowSEM,{data,observed,latentvars,eqs}~Join~os~Join~{onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True,
        InitValsFromSimplifiedModel -> True,Title->"F^a(2)",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],

     Apply[slowSEM,{data,observed,latentvars,eqs}~Join~os~Join~{onlyInitStep -> False, LastStep -> 6, onlyFirstStep -> True,
        InitValsFromSimplifiedModel -> True,Title->"F^a(3)",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],

      Apply[slowSEM,{data,observed,latentvars,eqs}~Join~os~Join~{LastStep->4,onlyFirstStep->False,VarCutFactor->3,InitPV->
      0.001,useNewWF->0.1,Title->"S2",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],

      Apply[slowSEM, {data, observed, latentvars, eqs}~Join~os~Join~{InitPV -> 1, onlyFirstStep -> True, Title -> "S2b",
                      calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],

      Apply[slowSEM, {data, observed, latentvars, eqs}~Join~os~Join~{LastStep -> 4, onlyFirstStep -> True, VarCutFactor -> 3,
                      InitPV -> 1, useNewWF -> 0.1, Title -> "S3",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}],
      Apply[slowSEM, {data, observed, latentvars, eqs}~Join~os~Join~{LastStep -> 4, onlyFirstStep -> True, VarCutFactor -> 3,
          InitPV -> 0.001, useNewWF -> 0.1, Title -> "S4",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}] 
          (*, Apply[slowSEM,{data,observed,latentvars,eqs,LastStep->4}~Join~os~Join~{Title->"S5",calcDataFit->OptionValue@useDataFit,calcDataFitWeighted ->OptionValue@calcDataFitWeighted}] *)  }];
      meanGOF=Mean[Map[#[OptionValue@GOFMethod]&,res]];
      GRES=res;
      res=Select[res,(#["ConvergenceOK"]&&#[OptionValue@GOFMethod]<meanGOF) &];
   (*First@*)SortBy[res, If[OptionValue@useDataFit,#["dataFit"] &,#[OptionValue@GOFMethod] &]]];
 
(******************* FAST ***************************************)

(* Implementation optimized for speed, but with restricted options
  W1: standard options
  Wn: weightsByLatentCount -> True
  Ww(Wn): weightsFromFirstRound -> True, weightsByLatentCountRound0 -> True


*)

    Options[fastSEM] = {MaxIter -> 500,  Title->"",ErrorMean0->0, (* -1: hard constraint *)
    userConstr -> {}, userLatentConstr -> {}, (* userLatentConstr is added for each case *)
   vectorConstr -> {}, (* Expressions in errors or latent variables, e.g. Dot[lv,err]==0 *)
   vectorOptim -> {}, (* Added to F - potentially very slow! *) 
   autoVectConstr -> 1000, (*   Add automatically vactor constraints e1.e2== 0 on error vars of 
     equations with at least this number of  latent variables  - 1000 -> ignore *)
   autoVectPenalty -> 1000, autoPF -> 1, (* the same but as Penalty *)
   LatentMean0 -> False, LatentMinVar->{}, (* vector of length latent *)
   weightExpo -> 1, InitValues -> {}, 
   AutoInit -> False, Verbose -> False,Quiet -> False, ShowWarnings -> False, ShowEqs->False,
   calcDataFit -> -1,(*-1: no, 0=unweighted, 1 weigted, 2, weighte by mean *)
  calcDataFitWeighted -> False, 
   absStat -> 0, L2Stat -> 1, (* in most cases, absStat>0 ist useless *)
   watchParas -> {}, userWeights -> {}, 
   weightsFromFirstRound -> False, doIterStep->0,Itereps->0.001,
   weightsByLatentCountRound0 -> False, weightsByLatentCount -> False,
    LatentCountExpo -> 0.5, Prec -> 5};

weight =.; (* Used as global variable *)

fastSEM["W1",data_, observed_, latentvars_, eqs_, OptionsPattern[]] := fastSEM[data, observed, latentvars, eqs,Quiet->True,
                      InitValues->OptionValue@InitValues,
   userConstr -> OptionValue@userConstr, userLatentConstr -> OptionValue@userLatentConstr,
   LatentMean0->OptionValue@LatentMean0,Quiet->OptionValue@Quiet,
   calcDataFit->OptionValue@calcDataFit,doIterStep->OptionValue@doIterStep];
fastSEM["Wn",data_, observed_, latentvars_, eqs_, OptionsPattern[]] := fastSEM[data, observed, latentvars, eqs,weightsByLatentCount -> True,
                      InitValues->OptionValue@InitValues,
   Quiet->True,LatentMean0->OptionValue@LatentMean0,calcDataFit->OptionValue@calcDataFit,
   userConstr -> OptionValue@userConstr, userLatentConstr -> OptionValue@userLatentConstr,doIterStep->OptionValue@doIterStep];
fastSEM["Ww(Wn)",data_, observed_, latentvars_, eqs_, OptionsPattern[]] := 
    fastSEM[data, observed, latentvars, eqs,weightsFromFirstRound -> True, 
    weightsByLatentCountRound0 -> True,Quiet->True,calcDataFit->OptionValue@calcDataFit,
                      InitValues->OptionValue@InitValues,
   userConstr -> OptionValue@userConstr, userLatentConstr -> OptionValue@userLatentConstr,
   LatentMean0->OptionValue@LatentMean0, doIterStep->OptionValue@doIterStep,Quiet->OptionValue@Quiet];


fastSEM[m_String,data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=
  Apply[fastSEM,Join[{data, observed, latentvars0, eqs0,Quiet->True,Title->m,calcDataFit->OptionValue@calcDataFit,
                        InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,doIterStep->OptionValue@doIterStep,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,Itereps->OptionValue@Itereps},
   Switch[m,
            "W1",{},
            "I20",{doIterStep -> 20,Itereps->0.0},
            "I-20",{doIterStep -> -20,Itereps->0.0},
            _,{}
     ]
  ]];





fastSEM[data_, observed_, latentvars_, eqs_, OptionsPattern[]] := 
  Module[{m = Length[eqs], n = Length[data], ok, i, i1, j, k, M, w, 
    StandardDeviation0, Variance0,t0 = AbsoluteTime[],
    F = 0, F000=0, Fvars, prules = {},  userConstrVals, 
    latvarE, latvarEVal, constr = OptionValue[userConstr], obsrules, 
    EQ, Rhs, Lhs, ListOfRules, eltable, abseps, MyAbs, 
    initvalrules, AllEEs = {}, GetAllVariables, getAllVariables, 
    tdata, errvars = Last /@ eqs, val0 = {}, res0, freeparas, indexOf,
     lvv, NonNull, headlist, Vals, lv, res, lastsol,  Errors={},NormV,
    neww, lat0rules, myDot, Correlation0,Covariance0, sol, sol0, lvprod, 
    weightvector, allVectConstr = OptionValue@vectorConstr, 
    quoteData = Count[(FreeQ[#1, Null] &) /@ Flatten[data], True]/(Length[data]*Length[First[data]])},
   ListOfRules[L_] :=  L == {} || (TrueQ[Head[L] == List] &&  TrueQ[Union@Map[Head, L] == {Rule}]);
   val0 = {}; NormV[L_]:= L/Apply[Plus,L];
    If[OptionValue@Quiet,Off[FindMinimum::sszero];Off[FindMinimum::lstol];Off[FindMinimum::bddir];
                 Off[FindMinimum::eit];Off[FindMinimum::sdir]; Off[FindMinimum::infeas];
                 Off[FindMinimum::cvmit]; Off[FindMinimum::steperr];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::cvmit]];

   If[OptionValue@AutoInit || OptionValue@weightsFromFirstRound || 
     OptionValue@weightsByLatentCountRound0,
    If[OptionValue@Verbose,Print["Recursion >"]];     
      res0 = 
     fastSEM[data, observed, latentvars, eqs, absStat -> 0, 
      AutoInit -> False,InitValues->OptionValue@InitValues, userConstr -> OptionValue@userConstr,
        userLatentConstr -> OptionValue@userLatentConstr, 
      weightsByLatentCount -> OptionValue@weightsByLatentCountRound0,
       LatentCountExpo -> OptionValue@LatentCountExpo, Quiet->OptionValue@Quiet,
      Prec -> OptionValue@Prec];
      If[OptionValue@Quiet,Off[FindMinimum::sszero];Off[FindMinimum::lstol];Off[FindMinimum::bddir];
                 Off[FindMinimum::eit];Off[FindMinimum::sdir]; Off[FindMinimum::infeas];
                 Off[FindMinimum::cvmit]; Off[FindMinimum::steperr];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::cvmit]];
    If[res0["ConvergenceOK"] && ListOfRules[res0["sol"]], 
     sol0 = val0 = res0["sol"],
     (* else *) 
     If[ListOfRules[res0["sol"]], sol0 = val0 = res0["sol"],  sol0 = val0 = {} ] ] ];
     If[OptionValue@Verbose&&(OptionValue@AutoInit || OptionValue@weightsFromFirstRound || 
     OptionValue@weightsByLatentCountRound0), Print[{"Round 1 done", OptionValue@watchParas /. sol0 }]];
    If[OptionValue@Verbose,Print["Init 1 done"]]; 
   abseps = 0.0001; MyAbs[x_] := Sqrt[x^2 + abseps];
   tdata = Transpose@data;
   StandardDeviation0[L_] :=  If[Length[L]<=1,0,StandardDeviation[Select[L, Not[TrueQ[Null == #]] &]]];
   Variance0[L_] := Variance[Select[L, FreeQ[#, Null] &]];
   indexOf[vars_, Name_] :=  Module[{r = 0, ii},  Do[If[Name == vars[[ii]], r = ii], {ii, 1, Length[vars]}]; r];
   Vals[v_] :=  Module[{p = indexOf[observed, v], ii},  Table[data[[ii]][[p]], {ii, 1, n}]];
   lvv[v_] := Module[{ii}, Table[v[ii], {ii, 1, n}]];
   NonNull[L_] := Select[L, TrueQ[FreeQ[#1, Null]] &];
   myDot[a_, b_] :=    Module[{kk}, 
     Sum[If[FreeQ[a[[kk]], Null] && FreeQ[b[[kk]], Null],  a[[kk]]*b[[kk]], 0], {kk, 1, Length[a]}]];
   Correlation0[A_, B_] := 
    Module[{a = {}, b = {}, ii}, 
     If[Length[A] != Length[B], Return["Err"]];
     Do[If[TrueQ[NumberQ[A[[ii]]] && NumberQ[B[[ii]]]], 
       AppendTo[a, A[[ii]]]; AppendTo[b, B[[ii]]]], {ii, 1, Length[A]}];
     If[Length[a] > 1, 
      If[StandardDeviation0[a]*StandardDeviation0[b] < 1/10^9, 
       "Novar", Correlation[a, b]], "NoData"]];
   Covariance0[A_, B_] :=   Module[{a = {}, b = {}, ii}, 
     If[Length[A] != Length[B], Return["Err"]];
     Do[If[TrueQ[NumberQ[A[[ii]]] && NumberQ[B[[ii]]]], 
       AppendTo[a, A[[ii]]]; AppendTo[b, B[[ii]]]], {ii, 1, Length[A]}];
     If[Length[a] > 1, Covariance[a, b], "NoData"]];
   Rhs[(a_) == (b_)] := b; Lhs[(a_) == (b_)] := a;
   headlist = {Or, And, Equal, Unequal, Less, LessEqual, Greater,  GreaterEqual, Inequality};
   getAllVariables[(f_)?NumericQ] := {};
   getAllVariables[{}] := {};
   getAllVariables[a_ -> b_] := 
    Union[getAllVariables[a], getAllVariables[b]];
   getAllVariables[t_] /; MemberQ[headlist, t] := {};
   getAllVariables[ll_List] := 
    Union[Flatten[Union[(getAllVariables[#1] &) /@ ll]]];
   getAllVariables[Derivative[n_Integer][f_][arg__]] := 
    getAllVariables[{arg}];
   getAllVariables[(f_Symbol)[arg__]] := 
    Union[{Module[{fvars}, 
       If[MemberQ[Attributes[f], NumericFunction] || 
         MemberQ[headlist, f], fvars = getAllVariables[{arg}], 
        fvars = f[arg]]; fvars]}]; 
   getAllVariables[other_] := {other};
   GetAllVariables[a_] := Union[Flatten[getAllVariables[a]]];
    If[OptionValue@Verbose,Print["Helper functions defined"]]; 
   obsrules = 
    Flatten[Table[
      Table[Rule @@ {observed[[j]][i], data[[i, j]]}, {i, 1,  Length[data]}], {j, 1, Length[observed]}]];
   freeparas =   Complement[GetAllVariables[First /@ eqs], Union[observed, latentvars, (#1[[2]] &) /@ eqs]];
   val0 = Join[val0, OptionValue@InitValues, ({#1, 0.1} &) /@ freeparas];
   Do[val0 =  Join[val0, Table[{lv[i], RandomReal[]/100}, {i, 1, n}]], {lv, latentvars}];
       If[OptionValue@Verbose,Print["Init 3 done"]]; 

   (*latvar[lv]=Varianz latent Variable;
   latvarE Variabe für apriori schätzung latvarEVal=
   Liste der apriori schätzungen*)
   EQ[{eq_, err_}] := 
    Module[{EE, EEs, manifest, Faktor, vars, ec,e}, 
     EE = Lhs[eq] - Rhs[eq]; vars = GetAllVariables[EE];
     manifest = 
      Intersection[observed, Flatten[Complement[vars, latentvars]]];
     EEs = 
      Table[EE /.   
        Union[Table[
          manifest[[j]] -> 
           data[[i]][[indexOf[observed, manifest[[j]]]]], {j, 1, 
           Length[manifest]}], 
         Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1, 
           Length[latentvars]}]], {i, 1, n}];
     EEs=ReleaseHold@EEs;
     If[err === Null||err===0,  
      Map[AppendTo[constr, #==0] &, EEs]; AppendTo[AllEEs, Table[0, n]],
      (* else with name *)
      If[OptionValue@ErrorMean0==-1,AppendTo[constr,0==Sum[e,{e,NonNull@EEs}]]];
      If[OptionValue@ShowEqs,Print[{"Adding to F",weight[err]*
         Sum[If[FreeQ[EEs[[i]], Null], 
           OptionValue@absStat*MyAbs[EEs[[i]]] + 
            OptionValue@L2Stat*EEs[[i]]^2, 0], {i, 1, n}]}]];
       F000=F000+ Sum[If[FreeQ[EEs[[i]], Null], 
           OptionValue@absStat*MyAbs[EEs[[i]]] + 
            OptionValue@L2Stat*EEs[[i]]^2, 0], {i, 1, n}];
       F = 
       F + weight[err]*
         Sum[If[FreeQ[EEs[[i]], Null], 
           OptionValue@absStat*MyAbs[EEs[[i]]] + 
            OptionValue@L2Stat*EEs[[i]]^2, 0], {i, 1, n}]; 
      AppendTo[AllEEs, EEs]; 
      AppendTo[Errors, Apply[ Rule, {err, Map[If[FreeQ[#, Null], #, Null] &, EEs]} ] ] ;] ];
   EQ /@ eqs;
 
  If[Length@OptionValue@LatentMinVar>0,
    Do[AppendTo[val0,{latmean[latent[[k]]],0.1}];
      AppendTo[constr,n*latmean[latent[[k]]]==Sum[latent[[k]][i],{i,1,n}] ];
      AppendTo[constr,n*OptionValue[LatentMinVar][[k]]<Sum[(latent[[k]][i]-latmean[latent[[k]]])^2 ,{i,1,n}]    ],
    {k,1,Length@latent}]
    ];

   constr = 
    Join[constr, 
     Select[Flatten[
       Table[OptionValue@userLatentConstr /. 
         Join[Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1,  Length[latentvars]}],                                 
              Table[eqs[[j, 2]] -> AllEEs[[j, i]], {j, 1, m}]], {i, 1,  n}]], FreeQ[#, Null] &]];
   If[OptionValue@autoVectConstr < 1000,
      Do[
     Do[ If[OptionValue@autoVectConstr <= 
        Length@Intersection[GetAllVariables[eqs[[i]]], 
          GetAllVariables[eqs[[j]]], latentvars],
       AppendTo[allVectConstr, 
        eqs[[i, 2]] . eqs[[j, 2]] == 0   ] ], {i, j + 1, m}], {j, 1, 
      m - 1} ] ];
   constr =  NonNull@Join[constr, 
     Map[  (# /. 
         NonNull@Join[
          Table[observed[[j]]->Transpose[data][[j]] ,{j,1,Length@observed}],
          Table[If[eqs[[j, 2]]===0,Null,eqs[[j, 2]] -> AllEEs[[j]]], {j, 1, m}], 
          Table[latentvars[[j]] -> lvv[latentvars[[j]]], 
              {j, 1, Length@latentvars}]]) & ,  ReleaseHold@allVectConstr] ];
   If[OptionValue@Verbose,
   Print["autoVectConstr added"]]; 
   If[OptionValue@autoVectPenalty < 1000,
    Do[Do[ 
      If[OptionValue@autoVectPenalty <= 
        Length@Intersection[GetAllVariables[eqs[[i]]], 
          GetAllVariables[eqs[[j]]], latentvars],
       F =    F + OptionValue@autoPF*((eqs[[i, 2]] . eqs[[j, 2]])^2    /. 
            Join[Table[eqs[[jj, 2]] -> AllEEs[[jj]], {jj, 1, m}],
                                                       
             Table[latentvars[[jj]] -> lvv[latentvars[[jj]]], {jj, 1, 
               Length@latentvars}] ] )   ], {i, j + 1, m}], {j, 1, 
      m - 1} ]];
   F = F + 
     Apply[Plus, 
      Map[  (# /. 
          Join[Table[eqs[[j, 2]] -> AllEEs[[j]], {j, 1, m}], 
           Table[latentvars[[j]] -> lvv[latentvars[[j]]], {j, 1, Length@latentvars}]]) & , OptionValue@vectorOptim]  ] ;
    
   If[OptionValue[Verbose], Print["EQS added"]];
   If[OptionValue@LatentMean0, constr = Join[constr, (Plus @@ lvv[#1] == 0 &) /@ latentvars]];
   
   F = F /. (OptionValue@userWeights);
   varmin = 0.0001; weightvector = {};
   If[OptionValue@weightsFromFirstRound && Length@sol0 > 0,
      Do[v0 = Variance0[ AllEEs[[j]] /. sol0];  
     AppendTo[weightvector, If[v0 > varmin, 1/v0, 1/varmin]];
     F = F /. weight[eqs[[j, 2]]] -> Last[weightvector], {j, 1, m}];
     If[OptionValue@Verbose,Print[{"weights applied:",weightvector}]] ];
   If[OptionValue@weightsByLatentCount,
    Do[ AppendTo[weightvector, 
      1/n^(OptionValue@LatentCountExpo*
          Length[Intersection[latentvars, 
            GetAllVariables[eqs[[j]]]]])];
     F = F /. weight[eqs[[j, 2]]] -> Last[weightvector], {j, 1, m}]];
   F = F /. weight[_] :> 1;
   If[Length@weightvector == 0, weightvector = Table[1, {m}]];
   
   Fvars = GetAllVariables@{F, constr};
   selvars = Select[val0, MemberQ[Fvars, First[#]] &];
   selvars0 = 
    Join[selvars, 
     Map[{#, 0.1} &, Complement[Fvars, Map[First, selvars]]]];
   val0 = {}; 
   Do[If[FreeQ[val0, First@p], AppendTo[val0, p]], {p, selvars0}];
   val0 = Map[{First@#, Last@#} &, val0];
   
   If [0 < Length@Complement[Fvars, Map[First, val0]], 
    Print[{"Warning: Variables without init value", 
      Complement[Fvars, Map[First, val0]]}]];
   If [0 < Length@Complement[Map[First, val0], Fvars], 
    Print[{"Warning: Variables fromt init value missing in F", 
      Complement[Map[First, val0], Fvars]}]];
   ok = Check[
     sol = FindMinimum[{F, constr}, val0, 
       MaxIterations -> OptionValue@MaxIter, 
       AccuracyGoal -> OptionValue@Prec, 
       PrecisionGoal -> OptionValue@Prec]; True, False];
   If[! ok,
            
    Do[val0[[i, 2]] = val0[[i, 2]]*RandomReal[{0.5, 1.5}], {i, 1, 
      Length@val0}];


   If[OptionValue@Verbose,Print["Now Minimizing"];GFminCommand=FindMinimum0[{F, constr}, val0,  MaxIterations -> OptionValue@MaxIter, 
        AccuracyGoal -> OptionValue@Prec, PrecisionGoal -> OptionValue@Prec]];

    ok = Check[sol = FindMinimum[{F, constr}, val0,  MaxIterations -> OptionValue@MaxIter, 
        AccuracyGoal -> OptionValue@Prec, PrecisionGoal -> OptionValue@Prec]; True, False];
    ];
   If[Not[ListOfRules[sol[[2]]]], 
    Print["FindMinimum gave no results"]; 
    FailedCall = 
     FindMinimum0[{F, constr}, val0, 
      MaxIterations -> OptionValue@MaxIter, 
      AccuracyGoal -> OptionValue@Prec, 
      PrecisionGoal -> OptionValue@Prec]; Return[Fail] ];
   If[! ok && OptionValue[Verbose], Print["Convergence Warning"]];
   If[OptionValue[Verbose], Print[{"Optim done",OptionValue@watchParas /. sol[[2]] }]];


   If[OptionValue[Verbose], Print["Considering Iteration"]];

  If[OptionValue@doIterStep>0,
     If[OptionValue[Verbose], Print["Iteration+"]];  
       Module[{solpre,EEoff,fails=0,ErrOK1,ErrOK2,ww1,ww2,dec,sol1,sol2,sol0,F0,FS,infolist,lastsol,k,s,EO0,EO1,EO2,lastEO,WRstep=0.2,
       ww=Table[1,Length@eqs],ve0,GoF,datT=Transpose@data},
            EEVs=Map[Apply[Plus,Map[#^2&,NonNull@#]]&,AllEEs]; 
            ww=NormV[ww]/2+1/2*NormV[1/(EEVs/.sol[[2]])]; If[OptionValue[Verbose], Print[{"Weight vector set to",ww}]];
            FS[w_List]:=FindMinimum[{w.EEVs, constr}, val0,MaxIterations ->500];
            ve0=Map[Function[{er}, er[[2]]  ],Errors];
            EEoff[sol_]:= Module[{ve=ve0/. sol[[2]]  ,u,v },
                Apply[Plus, Map[#^2&,Flatten[ Table[Table[Correlation0[ve[[u]],ve[[v]]],{v,u+1,Length[ve]}],{u,1,Length[ve]-1}] ] ]  ]   ]; 
            GoF[sol_]:=Module[{AE=AllEEs/.sol[[2]],LL=(lvv[#1] /. sol[[2]] &) /@ latentvars},
            {   Mean@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Max@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Mean@Abs@Flatten@Table[Correlation0[LL[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@latentvars}],
                Mean@Abs@Flatten@Table[Correlation0[datT[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@observed}],
                Sum[ww[[j]]*1/n*Sum[If[NumberQ[AE[[j,i]]],AE[[j,i]]^2,0],{i,1,n}],{j,1,m}]}
             ] ;

            sol=FS[ww]; EO0=EEoff[sol];
              If[OptionValue[Verbose], Print[{"Start Loop",OptionValue@watchParas /.Last@FS@ww }]];
              Do[   lastEO=EEoff[sol];   fails=0;   Do[ 
                If[OptionValue[Verbose]&&s==1,Print[{k,"EEof=",EEoff[sol],"GoF",GoF[sol],"paras=",OptionValue@watchParas/.sol[[2]],Round[NormV[ww],0.01]}]];
             ww1=NormV[ ReplacePart[ww,s->ww[[s]]*(1-WRstep) ]];ww2=NormV[ ReplacePart[ww,s->ww[[s]]*(1+WRstep) ]];
             ErrOK1 =  Check[solpre = FS[ww1];True, False];
             If[ErrOK1,sol1=solpre,sol1=sol];
             If[!ErrOK1,(*Print["ERR1"];*) GARGS=InsertWeights[arguments, ww1]; GWWW=ww1; ];
             ErrOK2 =  Check[solpre = FS[ww2];True, False];
             If[ErrOK2,sol2=solpre,sol2=sol];
             If[!ErrOK2,(*Print["ERR2"];*) GARGS=InsertWeights[arguments, ww2]; GWWW=ww2; ];
             EO1=EEoff[sol1];EO2=EEoff[sol2];
             sol0=sol;ww0=ww; dec=False;
             If[ErrOK1 && ErrOK2 && EO1<EO0 && EO1<EO2  , dec=True; sol=sol1;ww=ww1;EO0=EO1 ];
            If[ErrOK1 && ErrOK2 && EO2<EO0 && EO2<EO1 , dec=True; sol=sol2;ww=ww2;EO0=EO2 ];
            If[!dec,fails=fails+1]  ,  {s,1,Length[eqs]}  ];
              If[Abs[EO0-lastEO]/lastEO<OptionValue@Itereps,
                        If[OptionValue[Verbose],Print["Stationary point seems to be apporached "]]; 
                       res["iterEnd"]="Near stationary point";Break[];];
            If[fails==Length@eqs, If[OptionValue[Verbose],Print[{"too many non-improving dimensions, round ",k}]; res["iterEnd"]="No incr dimension";Break[]]];
            If[OptionValue[Verbose],Print[{"Round",k,"EO=",EO0,"sol...=",Take[sol[[2]] ,Min[3,Length[sol[[2]]]]],"weights",Round[ww,0.001],"#fails",fails}]],
             {k, OptionValue[doIterStep]}];
     ];
  
  ];


  If[OptionValue@doIterStep<0,  
     If[OptionValue[Verbose], Print["Iteration-"]];
       Module[{solpre,EEoff,fails=0,ErrOK1,ErrOK2,ww1,ww2,dec,sol1,sol2,sol0,F0,FS,infolist,lastsol,k,s,EO0,EO1,EO2,lastEO,WRstep=0.2,
            ww=Table[1,Length@eqs],ve0,datT=Transpose@data,GoF},
            EEVs=Map[Apply[Plus,Map[#^2&,NonNull@#]]&,AllEEs]; 
            FS[w_List]:=FindMinimum[{w.EEVs, constr}, val0,MaxIterations ->500];
            ww=NormV[ww]/2+1/2*NormV[1/(EEVs/.sol[[2]])]; 
            ve0=Map[Function[{er}, er[[2]]  ],Errors];
            GoF[sol_]:=Module[{AE=AllEEs/.sol[[2]],LL=(lvv[#1] /. sol[[2]] &) /@ latentvars},
            {   Mean@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Max@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Mean@Abs@Flatten@Table[Correlation0[LL[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@latentvars}],
                Mean@Abs@Flatten@Table[Correlation0[datT[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@observed}],
                Sum[ww[[j]]*1/n*Sum[If[NumberQ[AE[[j,i]]],AE[[j,i]]^2,0],{i,1,n}],{j,1,m}]}
             ] ;
            (* EEoff[sol_]:= {m,1,Length@latentvars,2,1}.GoF[sol]; *)
            EEoff[sol_]:= {2,1,2,2,1}.GoF[sol]; 
            sol=FS[ww]; EO0=EEoff[sol];
            If[OptionValue[Verbose], Print[{"Start Loop",OptionValue@watchParas /.Last@sol }];
            Print[{"GF",GoF[sol]}] ];
              Do[   lastEO=EEoff[sol];   fails=0;   Do[ 
                If[OptionValue[Verbose]&&s==1,Print[{k, EEoff[sol],GoF[sol],OptionValue@watchParas/.sol[[2]],Round[NormV[ww],0.01]}]];
             ww1=NormV[ ReplacePart[ww,s->ww[[s]]*(1-WRstep) ]];ww2=NormV[ ReplacePart[ww,s->ww[[s]]*(1+WRstep) ]];
             ErrOK1 =  Check[solpre = FS[ww1];True, False];
             If[ErrOK1,sol1=solpre,sol1=sol];
             If[!ErrOK1,(*Print["ERR1"];*) GARGS=InsertWeights[arguments, ww1]; GWWW=ww1; ];
             ErrOK2 =  Check[solpre = FS[ww2];True, False];
             If[ErrOK2,sol2=solpre,sol2=sol];
             If[!ErrOK2,(*Print["ERR2"];*) GARGS=InsertWeights[arguments, ww2]; GWWW=ww2; ];
             EO1=EEoff[sol1];EO2=EEoff[sol2];
             sol0=sol;ww0=ww; dec=False;
             If[ErrOK1 && ErrOK2 && EO1<EO0 && EO1<EO2  , dec=True; sol=sol1;ww=ww1;EO0=EO1 ];
            If[ErrOK1 && ErrOK2 && EO2<EO0 && EO2<EO1 , dec=True; sol=sol2;ww=ww2;EO0=EO2 ];
            If[!dec,fails=fails+1]  ,  {s,1,Length[eqs]}  ];
              If[Abs[EO0-lastEO]/lastEO<OptionValue@Itereps,
                        If[OptionValue[Verbose],Print["Stationary point seems to be apporached "]]; 
                       res["iterEnd"]="Near stationary point";Break[];];
            If[fails==Length@eqs, If[OptionValue[Verbose],Print[{"too many non-improving dimensions, round ",k}]; res["iterEnd"]="No incr dimension";Break[]]];
            If[OptionValue[Verbose],Print[{"Round",k,"EO=",EO0,"sol...=",Take[sol[[2]] ,Min[3,Length[sol[[2]]]]],"weights",Round[ww,0.001],"#fails",fails}]],
             {k, -OptionValue[doIterStep]}];
     ];
  
  ];


   If[OptionValue[Verbose], Print["Build result"]];

   res = 
    Association["n" -> n, "data" -> data, "latentvars" -> latentvars, "Title"->OptionValue@Title,
     "eqs" -> eqs, "observed" -> observed,
     "paras" -> Complement[GetAllVariables[First /@ eqs], observed], 
     "ConvergenceOK" -> ok, "F" -> F, "constr" -> constr,"AllEEs"->AllEEs,
     "weights" -> weightvector, 
     "allVectConstr" -> allVectConstr];
   
   If[OptionValue[Verbose], Print["post weights"]];
   res["weightsPost"]={}; 
    Do[v0 = Variance0[ AllEEs[[j]] /. sol[[2]]];  
     AppendTo[res["weightsPost"], If[v0 > varmin, 1/v0, 1/varmin]], {j, 1, m}];
   res["weightsPostN"]=res["weightsPost"]/Apply[Plus,res["weightsPost"]];
   res["weightsN"] = res["weights"] /Apply[Plus,res["weights"]];

   If[OptionValue[Verbose], Print["vars"]];
   res["vars"] = GetAllVariables[Join[{F}, constr]]; 
   res["sol0"] = sol0;
   res["sol"] = sol[[2]];
   res["Fmin"] = 1/n*F000/.sol[[2]];
   res["Fmin0"] = F /. res["sol"];
   res["R"] = (F /. res["sol"])/(quoteData*n);
   res["R0"] = (F /. res["sol"])/(quoteData*n*m);
   If[OptionValue[Verbose], Print["parasol"]];
   res["parasol"] = 
    Select[sol[[2]], MemberQ[res["paras"], First[#1]] &];
   res["LLs"] = (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLmeans"] = Mean /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLsds"] = StandardDeviation0 /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["EEs"] = AllEEs /. sol[[2]];
   res["EVars"] = (Variance[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EMeans"] = (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EE0s"] = 
    Table[Table[AllEEs[[j, i]], {i, 1, n}], {j, 1, Length[errvars]}] /. 
     sol[[2]];

   If[OptionValue[Verbose], Print["LE calculation"]];
   res["LEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EEs"][[i, s]]], 
        AppendTo[dat, {res["EEs"][[i, s]], res["LLs"][[j, s]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]], {i, 1, m}, {j, 1, Length@latentvars}];
   res["DEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] && NumberQ[data[[s,j]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], data[[s,j]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, Length@observed}];
  
   If[OptionValue[Verbose], Print["EE calculation"]];
   res["EEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[
         dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1,  n}]; Correlation0 @@ Transpose[dat]], {i, 1, m}, {j, 1, m}];
   res["EEcovs"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EEs"][[i, s]]] && NumberQ[res["EEs"][[j, s]]], 
        AppendTo[dat, {res["EEs"][[i, s]], res["EEs"][[j, s]]}]], {s, 1, n}]; Covariance0 @@ Transpose[dat]],
         {i, 1, m}, {j, 1, m}];
   
   If[OptionValue[Verbose], Print["GOF calculation"]];
   res["GOFMC"]=GOFMC[res];
   res["GOF"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],
               "EECorMean",Mean@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Mean@Abs@Flatten@res["LEcors"],
               "DECorMean",Mean@Abs@Flatten@res["DEcors"],
               (*"weightPostweightAgreement",Norm[res["weightsPost"]/Apply[Plus,res["weightsPost"]]-
                                                res["weights"]/Apply[Plus,res["weights"]] ],*)
               "SDWeightErr",StandardDeviation0[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]}/."Novar"->0;     
 res["GOF1"]=Check[Mean[{m,1,1,1,0}*{ res["GOF"][[2]],res["GOF"][[4]],res["GOF"][[6]],res["GOF"][[8]],res["GOF"][[10]]}],Null];

  If[OptionValue[Verbose], Print["GOF2 calculation"]];

  res["GOFM"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Max@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Max@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
               "SDWeightErr",StandardDeviation0[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]}/."Novar"->0;     
  res["GOF2"]=Check[Mean[{m,1,1,1,0}*{ res["GOFM"][[2]],res["GOFM"][[4]],res["GOFM"][[6]],res["GOFM"][[8]],res["GOFM"][[10]]}],Null];



 
If[OptionValue[Verbose], Print["NormedParas calculation"]];
 
 res["NormedParas"]=calcNormedParas[res];


If[OptionValue[Verbose], Print["DF calculation"]];

  If[OptionValue@calcDataFit>=0, 
      res["dataFit"]=CalcDataFit[res,OptionValue@calcDataFit>0,OptionValue@calcDataFit],
      res["dataFit"]="Data fit not calculated" ] ;


If[OptionValue[Verbose], Print["DF done"]];

      If[OptionValue@Quiet,On[FindMinimum::lstol];On[FindMinimum::bddir];On[FindMinimum::eit];
                 On[FindMinimum::sdir]; On[FindMinimum::cvmit];On[FindMinimum::steperr];
                 On[FindMinimum::cvmit];On[FindMinimum::acceptlev];On[FindMinimum::infeas];On[FindMinimum::lstol];On[FindMinimum::cvmit]];

   res["eqssol"] = 
    Table[{eqs[[i, 1]] /. res["parasol"], res["EVars"][[i]]}, {i, 1, Length@eqs}];
   res["eqssolNorm"] = 
    Table[{eqs[[i, 1]] /. res["NormedParas"], res["EVars"][[i]]}, {i, 1, Length@eqs}]/.res["sol"];
   res];



   (******************* SLOW ******************)




   Options[slowSEM] = {MaxIter -> 500, Prec->5, Title->"",
   userLatentConstr -> {}, userConstr -> {},userOptim -> {},   
   weightPreferEqual -> False, ErrorMean0 -> -1, ErrorCor0 -> 1, 
   ErrorLatCor0 -> 1, ErrorDatCor0 -> 0, showCor -> False, LatentMean0 -> False,
   ErrorLatOnlyRelevant -> False, ErrorCorOnlyRelevant -> False, 
   weightExpo -> 1, alphaFactor -> 1,(*bringt nix*)
   LatCExpo -> 0,(*1 bringt eher nix,
   obwohl die erste schätzung besser wird*)LatCFactor -> 0, 
   EEfactor -> 1, weightCons -> 0, EFactor -> 5, F0EE -> False, doIterStep->0,
   InitValues -> {}, CorAbs -> False, alphaStart -> 1, InitValsFromSimplifiedModel->False,
   InitValsFromF0->False,InitValsFromF0Method->"W1",
   InitPV->1, WeightVarBound->0, (* in last step *)
   AutoInit -> True, Verbose -> False,Quiet -> False, ShowWarnings -> False, 
   calcDataFit -> -1,(*-1: no, 0=unweighted, 1 weigted, 2, weighte by mean *)
   calcDataFitWeighted -> False, WE -> -1, absStat -> False, minWeight -> 0, 
   watchParas -> {}, optIter -> 5, onlyFirstStep -> True, onlyInitStep -> False, 
   normWeight -> False, SqrtWeights0->False,SqrtWeights6->False, VarCutFactor->10, StepP->1,Itereps-> 0.0001,
   Ipfe -> 1, Ipfl -> 1, Ipfd -> 1, Iw -> 1, useNewW -> True, useNewWF->0.9,
    LastStep -> 0, WEexpo -> 1,WPenalty->10,F112->0.02};



(***************** Experimentell: Latente Variablen nicht normiert *********** *)



slowSEM[meth_String,data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=Module[{},
   Apply[slowSEM2,{meth,data, observed, latentvars0, eqs0,InitValsFromF0->OptionValue@InitValsFromF0,
                        Quiet->OptionValue@Quiet,Title->meth,calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}] ] /; MemberQ[{"flex","flex2","Normal","Normal1","Normal2","Normal2b","Normal3","Normal3b",
                                                                      "Normal4","Normal5", "Normal6","Normal7"},meth];

slowSEM["x",data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=Module[{res0,res1},
   res0=fastSEM[OptionValue@InitValsFromF0Method,data, observed, latentvars0, eqs0,Quiet->OptionValue@Quiet]; 
   res1=Apply[slowSEMxxx,{data, observed, latentvars0, eqs0,InitValues -> Map[{#[[1]], #[[2]]} &, res0["sol"]],InitValsFromF0->False,
                      OffDiagFactorE -> 10, OffDiagFactorL -> 1, OffDiagFactorD -> 1,
                        Quiet->True,Title->"x",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
      res1["xinitres"]=res0; res1 ];

slowSEM[m_String,data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=
  Apply[slowSEM,Join[{data, observed, latentvars0, eqs0,Quiet->True,Title->m,calcDataFit->OptionValue@calcDataFit,
                      LatentMean0 -> OptionValue@LatentMean0,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      doIterStep->OptionValue@doIterStep,
                      InitValues->OptionValue@InitValues,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,
                      Verbose->OptionValue@Verbose,Itereps-> OptionValue@Itereps,
                      WPenalty->OptionValue@WPenalty,Quiet->OptionValue@Quiet},
     Switch[m,
            "W^a(1)",{useNewW->False},
            "W^a(W1)",{InitValsFromF0 -> True},
            "W^a(Wn)",{InitValsFromF0 -> True,InitValsFromF0Method->"Wn"},
            "W^a(Ww)",{InitValsFromF0 -> True,InitValsFromF0Method->"Ww(Wn)"},
            "W^a(W1a)",{InitValsFromF0 -> True,useNewW->True,useNewWF->0.5},
         (*gut*)   "W^{2a0}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 0, useNewWF -> 0.5},
         (*gut*)   "W^{2a0-I}",{InitValsFromF0 -> False, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 0, useNewWF -> 0.5},
            "W^{2a00}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 11/2, onlyFirstStep -> True, WeightVarBound -> 0, useNewWF -> 0.5},
            "W^{2a000}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 11/2, F112->0.0001, onlyFirstStep -> True, WeightVarBound -> 0, useNewWF -> 0.5},
            "W^{2a0.1}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 0.1, useNewWF -> 0.5},
         (*gut*)  "W^{2a}",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->6},
         (*gut*)  "W^{2a-I}",{useNewW->True,useNewWF->0.5,InitValsFromF0->False, LastStep->6},
            "W^{2ax}",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->6,SqrtWeights6->True},
            "W^{2b}",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->6,StepP->0.0,WeightVarBound -> 0},
            "W^{2a1}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 1, useNewWF -> 0.5},
            "W^{2a10}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 10, useNewWF -> 0.5},
            "W^{2a100}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 100, useNewWF -> 0.5},
            "W^{2a1000}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 1000, useNewWF -> 0.5},
            "W^{2a1006}",(*bad!*){InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 6, onlyFirstStep -> True, WeightVarBound -> 100, useNewWF -> 0.5},
            "I5",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True, useNewWF -> 0.5,doIterStep -> 5, StepP ->  0.001,Itereps-> 0.000001},
            "I20",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True, useNewWF -> 0.5,doIterStep -> 20, StepP -> 0.001,Itereps-> 0.000001},
            "I20b",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True, useNewWF -> 0.5,doIterStep -> 20, StepP -> 0.01,Itereps-> 0.000001},
            "W^7",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->100},
            "W^7a",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->0.01},
            "W^7ab",{useNewW->True,useNewWF->0.9,InitValsFromF0->True, LastStep->7,StepP->0.0001},
            "W^7b",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->0.0}, (* good *)
            "W^7b100",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->0.0,WPenalty->100}, 
            _,{}
     ]
  ]];


latvar=.; latmean=.; 
slowSEM[data_, observed_, latentvars_, eqs_, OptionsPattern[]] := 
  Module[{m = Length[eqs], n = Length[data], ok, i, i1, j, k, w, 
    StandardDeviation0, Variance0,minw=0.001,makeMW,t0 = AbsoluteTime[],
    F, Fe=0,FE, FD, FL,Fsol,FsolDebug,prec=OptionValue@Prec, errmean,
    Fvars, prules = {}, 
     latvarE, latvarEVal, K0,GOF1i=10^10,
    constr , obsrules, EQ, Rhs, Lhs, OptionValueInitPV,
     eltable, abseps, MyAbs, MySqrt, initvalrules, AllEEs = {}, 
    GetAllVariables, getAllVariables, tdata,tdatamean, errvars = Last /@ eqs, 
    val0 = {}, freeparas, indexOf, lvv, NonNull, headlist, Vals, lv, NormV,
    res, lastsol, www, www1, neww,  lat0rules, 
    weight, quoteData = Count[(FreeQ[#1, Null] &) /@ Flatten[data], True]/(Length[data]*Length[First[data]]),
    FFList,FEList,FLList,FDList,GOFList}, 
    
        If[OptionValue@Quiet,Off[FindMinimum::lstol];Off[FindMinimum::bddir];Off[FindMinimum::eit];Off[FindMinimum::sdir];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];
                 Off[FindMinimum::sszero];Off[FindMinimum::steperr];
                 Off[FindMinimum::sszero];Off[FindMinimum::cvmit];Off[FindMinimum::infeas];Off[FindMinimum::cvmit]];
   If[OptionValue[Verbose], Print[{"Enter slowSEM Quiet=",OptionValue@Quiet}]];
      NormV[L_]:= L/Apply[Plus,L];
   makeMW[L_List]:= Module[{minw0=Mean[Abs@L]/OptionValue@VarCutFactor},(*Print[{"minwcor",L,Map[#<minw0&,L]}];*)
         Map[If[#<minw0,minw0,#]&,L]];  (* ???? *)
 


   constr=OptionValue@userConstr;

   NonNull[L_] := Select[L, TrueQ[FreeQ[#1, Null]] &];
   abseps = 0.0001; MyAbs[x_] := Sqrt[x^2 + abseps];MySqrt[x_] := Sqrt[x + abseps];
   tdata = Transpose@data;
   tdatamean=Map[Mean[NonNull[#]]&,tdata];
   StandardDeviation0[L_] := 
    StandardDeviation[Select[L, Not[TrueQ[Null == #]] &]];
   Variance0[L_] := Variance[Select[L, FreeQ[#, Null] &]];
   indexOf[vars_, Name_] := 
    Module[{r = 0, ii}, 
     Do[If[Name == vars[[ii]], r = ii], {ii, 1, Length[vars]}]; r];
   Vals[v_] :=    Module[{p = indexOf[observed, v], ii}, Table[data[[ii]][[p]], {ii, 1, n}]];
   lvv[v_] := Module[{ii}, Table[v[ii], {ii, 1, n}]];
   myDot[a_, b_] := 
    Module[{kk}, 
     Sum[If[FreeQ[a[[kk]], Null] && FreeQ[b[[kk]], Null], 
       a[[kk]]*b[[kk]], 0], {kk, 1, Length[a]}]];
   Correlation0[A_, B_] := 
    Module[{a = {}, b = {}, ii}, 
     If[Length[A] != Length[B], Return["Err"]];
     Do[If[TrueQ[NumberQ[A[[ii]]] && NumberQ[B[[ii]]]], 
       AppendTo[a, A[[ii]]]; AppendTo[b, B[[ii]]]], {ii, 1,  Length[A]}];
     If[Length[a] > 1, 
      If[StandardDeviation[a]*StandardDeviation[b] < 1/10^9, "Novar",  Correlation[a, b]], "NoData"]];
   Rhs[(a_) == (b_)] := b; Lhs[(a_) == (b_)] := a;
   headlist = {Or, And, Equal, Unequal, Less, LessEqual, Greater,  GreaterEqual, Inequality};
   getAllVariables[(f_)?NumericQ] := {};
   getAllVariables[{}] := {};
   getAllVariables[a_ -> b_] := 
    Union[getAllVariables[a], getAllVariables[b]];
   getAllVariables[t_] /; MemberQ[headlist, t] := {};
   getAllVariables[ll_List] :=  Union[Flatten[Union[(getAllVariables[#1] &) /@ ll]]];
   getAllVariables[Derivative[n_Integer][f_][arg__]] :=  getAllVariables[{arg}];
   getAllVariables[(f_Symbol)[arg__]] := 
    Union[{Module[{fvars}, 
       If[
        MemberQ[Attributes[f], NumericFunction] || 
         MemberQ[headlist, f], fvars = getAllVariables[{arg}], 
        fvars = f[arg]]; fvars]}]; 
   getAllVariables[other_] := {other};
   GetAllVariables[a_] := Union[Flatten[getAllVariables[a]]];

   If[OptionValue[Verbose], Print["Helper functions defined"]];


   obsrules = 
    Flatten[Table[
      Table[Rule @@ {observed[[j]][i], data[[i, j]]}, {i, 1, 
        Length[data]}], {j, 1, Length[observed]}]];
   freeparas = 
    Complement[GetAllVariables[First /@ eqs], 
     Union[observed, latentvars, (#1[[2]] &) /@ eqs]];
   val0 = Join[val0, ({#1, 0.1} &) /@ freeparas];
   Do[val0 = Join[val0, Table[{lv[i], RandomReal[]/100}, {i, 1, n}]], {lv, latentvars}];
   (*latvar[lv]=Varianz latent Variable;
   latvarE Variabe für apriori schätzung latvarEVal=
   Liste der apriori schätzungen*)
   EQ[{eq_, err_}] := 
    Module[{EE, EEs, manifest,  vars, ec}, 
     EE = Lhs[eq] - Rhs[eq]; vars = GetAllVariables[EE];
     manifest = 
      Intersection[observed, Flatten[Complement[vars, latentvars]]];
     EEs = 
      Table[EE /.   
        Union[Table[
          manifest[[j]] -> 
           data[[i]][[indexOf[observed, manifest[[j]]]]], {j, 1, 
           Length[manifest]}], 
         Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1, 
           Length[latentvars]}]], {i, 1, n}];
     If[err === Null||err===0, If[OptionValue@Verbose,Print["Adding zero error constraint"]]; 
      Map[AppendTo[constr, #==0] &, EEs]; AppendTo[AllEEs, Table[0, n]], 
      AppendTo[AllEEs, EEs]];];
   EQ /@ eqs;
   If[OptionValue[Verbose], Print["EQS added"]];
   If[OptionValue[ErrorMean0] == -1, 
    constr = Join[constr, (Plus @@ NonNull[#1] == 0 &) /@ AllEEs]];
   If[OptionValue[ErrorMean0] > -1, 
    constr = Join[constr, Table[Plus @@ NonNull[AllEEs[[i]]] == n*errmean[eqs[[i,2]]] ,{i,1, Length@AllEEs}]];
    val0 = Join[val0, Table[{errmean[eqs[[i,2]]],0} ,{i,1, Length@eqs}]]];
   If[OptionValue[ErrorMean0] > 0, 
    Fe = Fe + OptionValue[ErrorMean0]*
       Sum[Plus @@ NonNull[AllEEs[[i]]]^2, {i, 1, Length[eqs]}]];
   (*Für autoinit*)constr00 = constr;
   constr = Join[constr, Table[latmean[lv]*n == Sum[lv[i], {i, 1, n}], {lv, latentvars}]];
   If[OptionValue@LatentMean0,constr = Join[constr, Table[0 == Sum[lv[i], {i, 1, n}], {lv, latentvars}]]];
   constr = Join[constr, Table[latvar[lv]*(n - 1) == Sum[(lv[i]-latmean[lv])^2, {i, 1, n}], {lv, latentvars}]];
   constr = Join[constr, Table[latvar[lv] > 0.0001, {lv, latentvars}]];
   latvarEVal = Table[1, Length@latentvars];
   val0 = Join[val0, Table[{latvar[latentvars[[j]]], latvarEVal[[j]]}, {j, 1, Length@latentvars}]];
   val0 = Join[val0, Table[{latmean[latentvars[[j]]], 0}, {j, 1, Length@latentvars}]];
   If[OptionValue@WeightVarBound!=0,AppendTo[val0,{K0,1}];AppendTo[constr,K0>0.0001]];
   normV[v_] := v/Apply[Plus, v];
   constr = 
    Join[constr, 
     Select[Flatten[
       Table[OptionValue@userLatentConstr /. 
         Join[Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1,  Length[latentvars]}],                                 
              Table[eqs[[j, 2]] -> AllEEs[[j, i]], {j, 1, m}]], {i, 1,  n}]], FreeQ[#, Null] &]];
   Do[Do[
     AppendTo[constr, 
       If[OptionValue@ErrorMean0>-1,
       EVM[j, k] == (1/n)*myDot[(AllEEs[[j]]-errmean[eqs[[j,2]]]), (AllEEs[[k]]-errmean[eqs[[k,2]]]) ],
       EVM[j, k] == (1/n)*myDot[AllEEs[[j]], AllEEs[[k]] ]
      ] ];
     If[j == k, AppendTo[constr, EVM[j, j] > 0.0000000001]];
     AppendTo[val0, {EVM[j, k], 0.001 + RandomReal[]/10}], {j, k, m}], {k, 1, m}];
 If[OptionValue[Verbose], Print["EVM added"]];

   Do[Do[
     AppendTo[constr, 
       If[OptionValue@ErrorMean0>-1,
         ELM[j, k] == 
           1/(n - 1)*Sum[(latentvars[[j]][i]-latmean[latentvars[[j]]])*If[FreeQ[AllEEs[[k, i]], Null], (AllEEs[[k, i]]-errmean[eqs[[k,2]]]  ), 0], {i, 1, n}],   
       ELM[j, k] == 
       1/(n - 1)*Sum[(latentvars[[j]][i]-latmean[latentvars[[j]]])*If[FreeQ[AllEEs[[k, i]], Null], AllEEs[[k, i]], 0], {i, 1, n}]   
       ]];
     AppendTo[val0, {ELM[j, k], 0.001 + RandomReal[]/100}], {j, 1, Length@latentvars}], {k, 1, m}];
 If[OptionValue[Verbose], Print["ELM added"]];
          
   Do[Do[AppendTo[constr, 
      EDM[j, k] == 1/(n - 1)*Sum[If[FreeQ[tdata[[j, i]]*AllEEs[[k, i]], Null], 
         (tdata[[j, i]]-tdatamean[[j]])*(AllEEs[[k, i]]-If[OptionValue@ErrorMean0>-1,errmean[eqs[[k,2]]],0]), 0], {i, 1, n}]];
     AppendTo[val0, {EDM[j, k], 0.001 + RandomReal[]/100}], {j, 1,Length@tdata}], {k, 1, m}];
  If[OptionValue[Verbose], Print["EDM added"]];
  
  
   
   (* Durch die Noirmierung mit 2/(n-3)^2 sind alle summanden von FE,
   FD,FL mit Variuanz 1 *)
   FE = (n - 3)*2*Sum[Sum[EVM[j, k]^2/(EVM[j, j]*EVM[k, k]), {j, k + 1, m}], {k, 1,m - 1}];
   FL = (n - 3)*Sum[Sum[ELM[jj, kk]^2/(latvar[latentvars[[jj]]]*EVM[kk, kk]) , {jj, 1, Length@latentvars}], {kk, 1, m}];
   FD = (n - 3)*Sum[Sum[EDM[jj, kk]^2/((0.000001+Variance0[tdata[[jj]]])*EVM[kk, kk]) , {jj,1, Length@tdata}], {kk, 1, m}];
   FEList =(n - 3)*Table[Table[EVM[j, k]^2/(EVM[j, j]*EVM[k, k]), {j, k + 1, m}], {k, 1,m - 1}];
   FLList =(n - 3)*Table[Table[ELM[jj, kk]^2/(latvar[latentvars[[jj]]]*EVM[kk, kk]) , {jj, 1, Length@latentvars}], {kk, 1, m}];
   FDList =(n - 3)*Table[Table[EDM[jj, kk]^2/((0.000001+Variance0[tdata[[jj]]])*EVM[kk, kk]) , {jj,1, Length@tdata}], {kk, 1, m}];
   GOFList=Flatten[{FEList,FLList,FDList,m^2*Table[EVM[k, k], {k, 1, m}]}];
   FFList=Table[EVM[k, k], {k, 1, m}];
   GOFList2={Flatten[{FEList,FLList,FDList}],Flatten[m^2*Table[EVM[k, k], {k, 1, m}]]};

   GFE = FE; GFL = FL; GFD = FD; 
   
   F[w_List, pfe_, pfl_, pfd_] := (F0 = Fe+Sum[w[[k]]*If[OptionValue@absStat,MySqrt@EVM[k, k],EVM[k, k]], {k, 1, m}];  
                                   F0 + pfe*FE + pfl*FL + pfd*FD+Sum[(OptionValue@userOptim)[[j]]^2 ,{j,1,Length@OptionValue@userOptim}]);
   Fsol[w_List, pfe_, pfl_, pfd_, val0_] := (F1=F[w, pfe, pfl, pfd];
                  FindMinimum[{F[w, pfe, pfl, pfd], constr}, val0, MaxIterations->OptionValue@MaxIter][[2]]);
   FsolDebug[w_List, pfe_, pfl_, pfd_, val0_] := (F1=F[w, pfe, pfl, pfd];
                  If[OptionValue@Verbose,
                      Print["GFindMinimumCall asigned"];
                      GFindMinimumCall= FindMinimum0[{F[w, pfe, pfl, pfd], constr}, val0];];
                  FindMinimum[{F[w, pfe, pfl, pfd], constr}, val0, MaxIterations->OptionValue@MaxIter][[2]]);
  
  
    If[OptionValue@Iw >= 0, www = Table[OptionValue@Iw, m], 
    www = Table[(1 + m)/(1 + Length@latentvars), m]]; (* Iw<0: Heuristic that more equations per latent should reduce error - little effect *)
   pfe0 = OptionValue@Ipfe; pfl0 = OptionValue@Ipfl; 
   pfd0 = OptionValue@Ipfd; 

 If[OptionValue@InitValsFromSimplifiedModel,
    If[OptionValue@Verbose,Print["ENTERING recursion for InitValsFromSimplifiedModel"]];
    sol=slowSEM[data, observed, latentvars,  Select[eqs, (1 == Length@Intersection[latentvars, GetAllVariables[#]]) &], 
       onlyInitStep -> True,InitValues->OptionValue@InitValues, LastStep -> 0, onlyFirstStep -> True,InitValsFromSimplifiedModel ->False];
    If[!sol["ConvergenceOK"]&&!OptionValue@Quiet,Print["InitValsFromSimplifiedModel failed"]];   
    If[sol["ConvergenceOK"],sol=sol["sol"];
      Do[  If[NumberQ[First[val0[[j]]]/.sol],  val0[[j,2]]=First[val0[[j]]]/.sol   ],{j,1,Length[val0]}]
    ];
    If[OptionValue@Verbose,Print["LEAVING recursion for InitValsFromSimplifiedModel"]];
 
    ];   

    OptionValueInitPV=OptionValue@InitPV;

If[OptionValue@InitValsFromF0,
    If[OptionValue@Verbose,Print["ENTERING recursion to fastSEM for InitValsFromF0"]];
   sol0=fastSEM[OptionValue@InitValsFromF0Method,data, observed, latentvars, eqs, LatentMean0 -> False,
          userLatentConstr->OptionValue@userLatentConstr,
      userConstr->OptionValue@userConstr,InitValues->OptionValue@InitValues,Quiet->OptionValue@Quiet];
            If[OptionValue@Quiet,Off[FindMinimum::sszero];Off[FindMinimum::lstol];Off[FindMinimum::bddir];
                 Off[FindMinimum::eit];Off[FindMinimum::sdir]; Off[FindMinimum::infeas];
                 Off[FindMinimum::cvmit]; Off[FindMinimum::steperr];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::cvmit]]; 
    If[OptionValue@Verbose,Print["RETURNED from recursion to fastSEM for InitValsFromF0"]];
    If[!sol0["ConvergenceOK"]&&!OptionValue@Quiet,Print["InitValsFromF0 failed"]];   
    If[sol0["ConvergenceOK"],
      OptionValueInitPV=(sol0["F"] /. sol0["sol"]);  GOF1i=sol0["GOF1"];
      sol=sol0["sol"];
      If[OptionValue@Verbose, Print[{"val0 determined", Take[sol,Min[5,Length@sol]],OptionValue@watchParas /. sol}]];
      If[Head[sol]==List&&Union[Map[Head,sol]]=={Rule},
         Do[  If[NumberQ[First[val0[[j]]]/.sol],  val0[[j,2]]=First[val0[[j]]]/.sol   ],{j,1,Length[val0]}],
         If[True||!OptionValue@Quiet,Print["solution was not rule list"]]
      ]
    ];
 ];
If[OptionValue@Verbose, Print[{"val0 determined and initialised,val0"}]];

   
   Fvars = GetAllVariables@{F[www, pfe0, pfl0, pfd0], constr};
   selvars = Select[val0, MemberQ[Fvars, First[#]] &];
   selvars0 = Join[selvars,  Map[{#, 0.1} &, Complement[Fvars, Map[First, selvars]]]];
   selvars = {}; 
   Do[If[FreeQ[selvars, First@p], AppendTo[selvars, p]], {p,  selvars0}];
   GF = F[www, pfe0, pfl0, pfd0]; Gval0 = val0; GC = constr; GFsol=Fsol; Gwww=www; 

  Do[AppendTo[val0,{v,0}],{v,Complement[Fvars,Map[First,val0]]}];

  If[OptionValue@Verbose, 
    Print[{"Now 00"}]; Print[{"Fsol arguments:",www, OptionValueInitPV, OptionValueInitPV, OptionValueInitPV, 
       Take[val0,Min[5,Length@val0]]}]];


   ok = Check[sol00 = FsolDebug[www, OptionValueInitPV, OptionValueInitPV, OptionValueInitPV, val0]; True, False]; 
   If[!ok,ok = Check[sol00 = FsolDebug[www, OptionValueInitPV, OptionValueInitPV, OptionValueInitPV,
                                   Map[{#[[1]],#[[2]]+RandomReal[{0,0.1}]}&,val0]]; True, False]]; 
  If[OptionValue@Verbose, Print["sol00 determined"]];
   
   ok=ok&&TrueQ[Head[sol00]==List&&Union[Map[Head,sol00]]=={Rule}];

   If[OptionValue@Verbose, If[ok, Print[{"sol of init step found", OptionValue@watchParas /. sol00}],
   Print["Issues in 00"] ] ];
   If[ok, val0 = Map[{#[[1]], #[[2]]} &, sol00]];
   If[!ok&&!OptionValue@Quiet,Print["Conv issues in step 00"]];
   
   If[OptionValue@onlyInitStep,sol=sol00,

   If[ok,If[OptionValue@useNewW,  
       www =(OptionValue@useNewWF)*Table[1/Mean[Table[EVM[k, k], {k, 1, m}]/.sol00] , m];
       www=www+(1-OptionValue@useNewWF)*(1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol00]) ]]; (* ususally almost no improvement *)
   
   If[OptionValue@SqrtWeights0,www=Sqrt[www]];

   
   If[OptionValue@Verbose, Print[{"Init done, status,watchParas, weights",ok,If[ok,OptionValue@watchParas/.sol00,{}], www}]];

   ok = Check[sol = Fsol[www, pfe0, pfl0, pfd0, val0]; True, False];
   If[! ok, 
    val00 = Map[{First@#, Last[#] + RandomReal[{0.05, 0.01}]} &, val0];
    ok =  Check[sol = Fsol[www, pfe0, pfl0, pfd0, val00]; True, False];
    ];
   If[! ok, 
    val00 = Map[{First@#, Last[#] + RandomReal[{0.05, 0.01}]} &, val0];
    prec=1+Floor[OptionValue@Prec/2];
    ok =  Check[sol = Fsol[www, pfe0/10, pfl0/10, pfd0/10, val00]; True, False];
    prec=OptionValue@Prec
    ];
    ok=ok&&TrueQ[Head[sol]==List&&Union[Map[Head,sol]]=={Rule}];
   If[! ok &&!OptionValue@Quiet, Print[{"Conv issues in first step"}]];
   If[OptionValue@Verbose, Print[{ok,"First sol=", If[ok,OptionValue@watchParas /. sol,"not ok"]}]];
   ]; (* end of else-part of  of If[OptionValue@onlyInitStep*)

   If[Not[OptionValue@onlyFirstStep]&&Not[OptionValue@onlyInitStep],
    
    www1 = If[ok,1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol],www]; 
    If[OptionValue@Verbose, Print[{"SecondStep, weights",www1}]];
    If[OptionValue@normWeight, www1 = normV@www1];
    ok = Check[sol1 = Fsol[www1, pfe0, pfl0, pfd0, If[ok,Map[{First@#, Last@#} &, sol],val0]]; True, False];
    If[! ok, 
     val00 = Map[{First@#, Last[#] + RandomReal[{0.05, 0.01}]} &, val0];
     ok =  Check[sol1 =  Fsol[If[ok,1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol,www1]], pfe0, pfl0, pfd0,  val00]; True, False];
     ];
    If[! ok, If[!OptionValue@Quiet,Print[{"Conv issues in step 1"}]], (* else step ok *) sol = sol1];  
    ]; (* End only 1st step *) 

    If[OptionValue@Verbose&&Not[OptionValue@onlyFirstStep], 
          Print[{"Now optional steps",ok,If[ok,OptionValue@watchParas/.sol,"not ok"]}]];
   
   (* Experiment Beginn - das bringt nix  *)
   If[OptionValue@Verbose,Print@"Now Starting Phases 2,3"];
    weightCond = True;
   If[ (OptionValue@LastStep == 1 ||  OptionValue@LastStep == 3), 
    ok3 = Check[
      sol3 = FindMinimum[{F[Table[ow[j], {j, 1, m}], 1, 1, 1] + 
                                  If[weightCond, n^OptionValue@WEexpo*Sum[(1 - ow[j]*EVM[j, j])^2, {j, 1, m}], 0],                                                               
                  Join[constr, {Sum[ow[j], {j, 1, m}] >  Apply[Plus, 1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]]},
                  Table[ow[j] > 0, {j, 1, m}] ]},
           Join[val0, Table[{ow[j], If[(EVM[j, j]/.sol) <minw,1/minw,1/EVM[j, j] /. sol]}, {j, 1, m}]]][[2]]; True, False];
    oww = Table[ow[j], {j, 1, m}] /. sol3; 
    If[ok3, sol = sol3;www=oww];
    oww = oww/Apply[Plus, oww];
    If[OptionValue@LastStep == 3, 
     ok3 = Check[sol3b = 
        Fsol[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol3], pfe0, pfl0, pfd0, Map[{First@#, Last@#} &, sol3]]; True, False];
     ];
    If[ok3, sol = sol3b;www=1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol3]] ];
   
   (* Experimene Ende *)
   (* Experiment 2 Beginn - das bringt nix  *)
   If[OptionValue@Verbose,Print@"Now Starting Last Phases"];
    If[ OptionValue@LastStep == 2, 
    weightCond = True;
    ok3 = 
     Check[sol3b = 
       FindMinimum[{F[Table[ow[j], {j, 1, m}], 1, 1, 1]/ Sqrt@Sum[(1/ow[j])*EVM[j, j], {j, 1, m}] +  
                    n^2*Sum[(1 - ow[j]*EVM[j, j])^2, {j, 1, m}],                                            
          Join[constr, {Sum[ow[j], {j, 1, m}] >  Apply[Plus, 1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol] ]},
           Table[ow[j] > minw, {j, 1, m}] ]},
         Join[val0, Table[{ow[j],    If[(EVM[j, j]/.sol) <minw,1/minw,1/EVM[j, j] /. sol]    }, {j, 1, m}]]][[2]]; True, False];
    oww = Table[ow[j], {j, 1, m}] /. sol3b; 
    oww = oww/Apply[Plus, oww]; 
    If[ok3, sol = sol3b; ok=ok3] ];
   (* Experiment 2 Ende *)

If[OptionValue@WeightVarBound>0,  
    Fsol[w_List, pfe_, pfl_, pfd_, val0_] :=  
       FindMinimum[{F[w, pfe, pfl, pfd]+OptionValue@WeightVarBound*Sum[(w[[j]]*EVM[j,j]-K0)^2,{j,1,m}], constr}, val0,
           MaxIterations->OptionValue@MaxIter (*,AccuracyGoal->prec,  PrecisionGoal->prec *)][[2]]
];
If[OptionValue@WeightVarBound<0,  
    Fsol[w_List, pfe_, pfl_, pfd_, val0_] :=  
       FindMinimum[{F[w, pfe, pfl, pfd], 
                    Join[constr,If[OptionValue@WeightVarBound<0,Table[w[[j]]*EVM[j,j]==K0,{j,1,m}],{}]]}, val0,
           MaxIterations->OptionValue@MaxIter (*,AccuracyGoal->prec,  PrecisionGoal->prec *)][[2]]
];



    If[ OptionValue@LastStep == 4,  (* Identity was makeMW, but seem snot a good idea *)
     ok4 = Check[sol4 = Fsol[If[ok,1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol],www], pfe0, pfl0, pfd0, 
       If[ok,Map[{First@#, Last@#} &, sol],val0 ] ]; True, False]; 
    If[ok4, sol = sol4; ok=True] ];
    If[ OptionValue@LastStep == 5, 
     ok5 = Check[sol5 = Fsol[Identity[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]], 0, 0, 0, Map[{First@#, Last@#} &, sol]]; True, False]; 
    If[ok5, sol = sol5; ok=True] ];
     If[ OptionValue@LastStep == 11/2, 
     ok5 = Check[sol5 = Fsol[Identity[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]], OptionValue@F112, OptionValue@F112, OptionValue@F112, Map[{First@#, Last@#} &, sol]]; True, False]; 
    If[ok5, sol = sol5; ok=True] ];
  If[ OptionValue@LastStep == 6, 
    If[OptionValue@Verbose,Print[{"LastStep6, weights",1/makeMW[Table[If[OptionValue@SqrtWeights6,Sqrt[EVM[k, k]],EVM[k, k]], {k, 1, m}] /. sol]}]];
     ok6 = Check[sol6 = Fsol[Identity[1/makeMW[Table[If[OptionValue@SqrtWeights6,Sqrt[EVM[k, k]],EVM[k, k]], {k, 1, m}] /. sol]], 
       OptionValue@StepP,OptionValue@StepP,OptionValue@StepP, Map[{First@#, Last@#} &, sol]]; True, False]; 
    If[ok6, sol = sol6; ok=True,If[!OptionValue@Quiet,Print["Warning: ok6 fail"]]] ];
  
 
    If[ OptionValue@LastStep == 7, 
     F7=Sum[wgt[k]*EVM[k, k], {k, 1, m}]+OptionValue@StepP*(FE + FL + FD)+Sum[(OptionValue@userOptim)[[j]]^2 ,{j,1,Length@OptionValue@userOptim}]+
       OptionValue@WPenalty*Length@data*Sum[(wgt[k]*EVM[k, k]-K0)^2,{k,1,m}];
     ok7 = Check[sol7 = FindMinimum[{F7, 
                                     Join[constr,{K0>0.001,1<Sum[wgt[k],{k,1,m}]},Map[(First@#>1/5*Last@#)&,Transpose@{Table[wgt[k],{k,1,m}],1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]} ],
                                       Map[(First@#<5*Last@#)&,Transpose@{Table[wgt[k],{k,1,m}],1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]} ] ]},  
                                    Join[{{K0,1}},Transpose@{Table[wgt[k],{k,1,m}],1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]},Map[{First@#, Last@#} &, sol ]] ]; True, False]; 
    If[ok7, sol = sol7[[2]]; ok=True] ];




If[OptionValue@doIterStep>0,

Module[{solpre,EEoff,fails=0,ErrOK1,ErrOK2,ww1,ww2,dec,sol1,sol2,sol0,F0,FS,infolist,lastsol,k,s,EO0,EO1,EO2,lastEO,WRstep=0.2,
            ww=normV[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]],ve0,datT=Transpose@data,GoF,varE,EEVL},
            EEVs=Map[Apply[Plus,Map[#^2&,NonNull@#]]&,AllEEs];
            varE[sol_]:= 1/n*EEVs/.sol;
            FS[w_List]:=Fsol[w/Mean[varE[sol]],  OptionValue@StepP,OptionValue@StepP,OptionValue@StepP, Map[{First@#, Last@#} &, sol]];
            ve0=Map[Function[{er}, er[[2]]  ],Errors];
            GoF[sol_]:=Module[{AE=AllEEs/.sol,LL=(lvv[#1] /. sol &) /@ latentvars},
            {   Mean@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Max@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Mean@Abs@Flatten@Table[Correlation0[LL[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@latentvars}],
                Mean@Abs@Flatten@Table[Correlation0[datT[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@observed}],
                Sum[ww[[j]]*1/n*Sum[If[NumberQ[AE[[j,i]]],AE[[j,i]]^2,0],{i,1,n}],{j,1,m}]}
             ] ;
            EEoff[sol_]:= {Length@eqs,1,1,1,1}.GoF[sol]; 
            sol=FS[ww]; EO0=EEoff[sol];
           If[OptionValue@Verbose, Print[{"Start Loop",OptionValue@watchParas /.sol }];
            Print[{"GF",GoF[sol]}]];
              Do[   lastEO=EEoff[sol];   fails=0;   Do[ 
                If[OptionValue@Verbose&&s==1,Print[{k, EEoff[sol],GoF[sol],OptionValue@watchParas/.sol,Round[NormV[ww],0.01]}]];
             ww1=NormV[ ReplacePart[ww,s->ww[[s]]*(1-WRstep) ]];ww2=NormV[ ReplacePart[ww,s->ww[[s]]*(1+WRstep) ]];
             ErrOK1 =  Check[solpre = FS[ww1];True, False];
             If[ErrOK1,sol1=solpre,sol1=sol];
             If[!ErrOK1,(*Print["ERR1"];*) GARGS=InsertWeights[arguments, ww1]; GWWW=ww1; ];
             ErrOK2 =  Check[solpre = FS[ww2];True, False];
             If[ErrOK2,sol2=solpre,sol2=sol];
             If[!ErrOK2,(*Print["ERR2"];*) GARGS=InsertWeights[arguments, ww2]; GWWW=ww2; ];
             EO1=EEoff[sol1];EO2=EEoff[sol2];
             sol0=sol;ww0=ww; dec=False;
             If[ErrOK1 && ErrOK2 && EO1<EO0 && EO1<EO2  , dec=True; sol=sol1;ww=ww1;EO0=EO1 ];
            If[ErrOK1 && ErrOK2 && EO2<EO0 && EO2<EO1 , dec=True; sol=sol2;ww=ww2;EO0=EO2 ];
            If[!dec,fails=fails+1]  ,  {s,1,Length[eqs]}  ];
              If[Abs[EO0-lastEO]/lastEO<OptionValue@Itereps,
                        If[OptionValue[Verbose],Print["Stationary point seems to be apporached "]]; 
                       res["iterEnd"]="Near stationary point";Break[];];
            If[fails==Length@eqs, If[OptionValue[Verbose],Print[{"too many non-improving dimensions, round ",k}]; 
            res["iterEnd"]="No incr dimension";Break[]]];
            If[OptionValue[Verbose],Print[{"Round",k,"EO=",EO0,"sol...=",Take[sol ,Min[3,Length[sol]]],"weights",Round[ww,0.001],"#fails",fails}]],
             {k, OptionValue[doIterStep]}];

] ];




   If[OptionValue@Verbose, Print[{"Starting report",If[ok,OptionValue@watchParas /. sol,"not ok"]}]];
   res = 
    Association["n" -> n, "data" -> data, "latentvars" -> latentvars, "Title"->OptionValue@Title,
     "eqs" -> eqs, "observed" -> observed, "ConvergenceOK"->ok,"val0"->val0,
     "paras" -> Complement[GetAllVariables[First /@ eqs], observed],
     "FEList"->FEList,"FLList"->FLList,"FDList"->FDList,"FFList"->  FFList];
   res["F"] = F1;
   res["Fmin"]=Sum[EVM[k, k], {k, 1, m}]/.sol;
   res["constr"] = constr; 
   res["vars"] = GetAllVariables[Join[{F1}, constr]];
   res["sol"] = sol;
   
   If[!ok&&!OptionValue@Quiet,Print["Warning: slowSEM did not succeed"];Return[res]];

   res["weights"] = www;
   res["weightsN"] = res["weights"] /Apply[Plus,res["weights"]]; 
   res["weightsPost"]=1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol];
   res["weightsPostN"] = res["weightsPost"] /Apply[Plus,res["weightsPost"]]; 
   res["Fmin0"] = F0 /. weight[_] :> 1/Sqrt[m] /. res["sol"];
   res["R"] = (F00 /. res["sol"])/(quoteData*n);
   res["R0"] = (F00 /. res["sol"])/(quoteData*n*m);
   res["parasol"] = Select[sol, MemberQ[res["paras"], First[#1]] &];
   res["LLs"] = (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLmeans"] = Mean /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLsds"] =  StandardDeviation /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["EEs"] = AllEEs /. sol;
   res["EVars"] = (Variance[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EEmeans"] = (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EMeans"] =  (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
    res["EE0s"] =  Table[Table[AllEEs[[j, i]], {i, 1, n}], {j, 1, Length[errvars]}] /.  sol;
   res["LEcors"] = Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], res["LLs"][[j, s]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
         {i, 1, m}, {j, 1, Length@latentvars}];
   res["DEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] && NumberQ[data[[s,j]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], data[[s,j]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, Length@observed}];
   res["EEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1,  n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, m}];
   res["EEcovs"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[
         dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1, 
        n}]; Covariance @@ Transpose[dat]], {i, 1, m}, {j, 1, m}];

   res["GOFMC"]=GOFMC[res];

    res["GOF"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Mean@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Mean@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
              (* "weightPostweightAgreement",Norm[res["weightsPost"]/Apply[Plus,res["weightsPost"]]-
                                                res["weights"]/Apply[Plus,res["weights"]] ], *)
                "SDWeightErr",StandardDeviation[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]};     
  res["GOF1"]=Check[Mean[{m,1,1,1,0}*Select[res["GOF"],NumberQ]],Null];
  If[GOF1i*1.5<res["GOF1"],res["ConvergenceOK"]=False;If[!OptionValue@Quiet,Print[{"GOF1 convergence warning",GOF1i,res["GOF1"],{b1,b2,b3}/.res["sol"]}]]];
(*!!!*)    res["GOF1"]=Mean[GOFList/.res["sol"]];
res["GOFL"]=Map[Mean,GOFList2/.res["sol"]];

   res["GOFM"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Max@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Max@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
               "SDWeightErr",StandardDeviation[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]};     
  res["GOF2"]=Check[Mean[{m,1,1,1,0}*Select[res["GOFM"],NumberQ]],Null];


 res["NormedParas"]=calcNormedParas[res];

   res["eqssol"] = 
    Table[{eqs[[i, 1]] /. res["parasol"], res["EVars"][[i]]}, {i, 1, Length@eqs}];
   res["eqssolNorm"] = 
    Table[{eqs[[i, 1]] /. res["NormedParas"], res["EVars"][[i]]}, {i, 1, Length@eqs}]/.res["sol"];

  If[OptionValue@calcDataFit>=0, 
      res["dataFit"]=CalcDataFit[res,OptionValue@calcDataFit>0,OptionValue@calcDataFit],
      res["dataFit"]="Data fit not calculated" ] ;


    res["time"]= AbsoluteTime[]-t0;
    If[OptionValue@Quiet,On[FindMinimum::lstol];On[FindMinimum::bddir];On[FindMinimum::eit];On[FindMinimum::sdir];
                 On[FindMinimum::cvmit];On[FindMinimum::acceptlev];On[FindMinimum::lstol];On[FindMinimum::cvmit];
                 On[FindMinimum::cvmit];On[FindMinimum::steperr];
                 On[FindMinimum::sszero];On[FindMinimum::infeas];On[FindMinimum::sszero];];

   res];





MeanRound[{}] :="NoData";
MyRound[Mean[{}]]:="NoData";
MeanRound[L_] :=  MyRound[If[Length@L>0,Composition[Mean, Select[#, NumberQ] &]@L,Null], 0.001] /; Head[L] == List || NumberQ[L]; 
MeanRound[L_String] := L; StandardDeviationRound[L_String] := L; 
StandardDeviation0[L_List]:= If[Length[Select[L, NumberQ]]<2,"DataLow", StandardDeviation[Select[L, NumberQ]]];
StandardDeviationRound[{}] :="NoData";
StandardDeviationRound[{a_}] :="DataLow";
StandardDeviationRound[L_] := 
 MyRound[If[Length@L>1,Composition[StandardDeviation0, Select[#, NumberQ] &]@L,Null], 0.001] /; 
  (Head[L] == List&&Length[L]>=2) || NumberQ[L]; 
doPrintR = True; 
PrintR[a_] := If[doPrintR, Print[MyRound[a, 0.001]]];


Options[SemTest] = {Verbose -> 0, printSummary -> True,   bollen -> False, Title -> "Titel", Digits -> 2,
                    ltx2Lines->True,ltx3Lines->True,ltxGOF->True,NormParas->{} ,absStat -> False, ErrorMean0->-1,
              normed->False,normedTrue->{},userLatentConstr -> {}, userConstr -> {},LatentMean0->False,
              LatInit->{}, (* a list like {1,5,9}  incating the indicdes of the overved variabes whose data schould be 
              used as initial Values for latent variables *)
              useGOFMC->True,
              ExportData->Null,stopOnMessage ->False,debug->False,calcDataFit->-1,MaxIter->250}; 
              (* genFun ist entweder Funktion m->Data oder Liste von Iters Daten *)
              
SemTest[Iters_, SampleSizeList_, genFun_, fastMeth_, slowMeth_, paras_, truevalues_, {obs_, lat_, eqs_}, OptionsPattern[]] :=
 Module[{  result = <||>, RRL, fails, bbres, k, kk=0,j, i,ii,kkk, times, gofs,dfs,t, res, rep,rmse,RMSE,
   datR, val0,m, ltx = "", p, LatexForm, digits = OptionValue[Digits],dRound,stop=False,datasets={},
   Mean0 = Composition[Mean, Select[#, NumberQ] &],SD0 = Composition[StandardDeviation, Select[#, NumberQ] &],
   mc = Length[fastMeth] + Length[slowMeth] +  If[OptionValue@bollen, 1, 0]},  
  RRL = Table[{}, mc]; fails = Table[0, mc]; times = Table[{}, mc];gofs = Table[{}, mc]; dfs = Table[{}, mc];
  dRound[x_]:=MyRound[x,0.1^digits];
  RMSE[L0_, t_] :=  Module[{k, L = Select[L0, NumberQ]}, N@Sqrt[1/Length[L]*Sum[(L[[k]] - t)^2, {k, 1, Length[L]}]]];
  LatexForm[S_] := Module[{s = ToString@S, c},
    c = Characters[s]; 
    If[Length@c == 2, s = c[[1]] <> "_" <> c[[2]]];
    "$" <> s <> "$"];
  ltx = "\\begin{table}
     \t\\caption{" <> OptionValue@Title <> " (" <> ToString[Iters] <> 
    " simulations for each $n$)" <> " }
    \t{\\footnotesize\\begin{tabular}{";
  Do[ltx = ltx <> "c", 2 + mc];
  ltx = ltx <> "}\\hline
";
  ltx = ltx <> "$n$ & var  "; 
  If[OptionValue@bollen, ltx = ltx <> " & ML "];
  Do[ltx = ltx <> " & $" <> StringReplace[m,{"W1"->"W_1","Wn"->"W_n"}]<> "$ ", {m, fastMeth}]; 
  Do[ltx = ltx <> " & $" <>StringReplace[m,{"W1"->"W_1","Wn"->"W_n"}] <> "$ ", {m, slowMeth}];
  ltx = ltx <> "\\\\ \t
";
  Do[If[OptionValue@Verbose > 0, 
    Print["--- n=" <> ToString@SampleSize]];
    If[stop,Break[]];
   Do[
    If[OptionValue@Verbose >= k,Print[k]];
    If[TrueQ[Head[genFun]==List],
         kk=kk+1;
         datR=genFun[[kk]], 
         datR = genFun[SampleSize]]; 
    If[Length@lat==Length@OptionValue@LatInit, 
        val0={}; 
        Do[AppendTo[val0,Table[{lat[[kkk]][ii],datR[[ii,OptionValue[LatInit][[kkk]]]]},{ii,1,Length@datR}]],{kkk,1,Length@lat}],
        val0={}]; (* InitValues *)
    AppendTo[datasets,datR];
    If[!TrueQ[OptionValue@ExportData==Null],
      Export[OptionValue@ExportData<>"dat_"<>ToString[Position[ SampleSizeList,SampleSize][[1,1]]]<>"_"<>ToString[k]<>".csv",Join[{obs},datR]]];
    j = 1;
    Check[
    If[OptionValue@bollen,
       {t, res} = AbsoluteTiming@Take[RBollen[datR, Bobserved], 3]; 
     AppendTo[RRL[[j]], res]; AppendTo[times[[j]], t];
     If[OptionValue@Verbose >= k, Print[{"lavaan", res}]];
      j = j + 1
     ];

    (* fast*)
    Do[{t, res} = AbsoluteTiming@fastSEM[m, datR, obs, lat, eqs,calcDataFit->OptionValue@calcDataFit,
             LatentMean0->OptionValue@LatentMean0,InitValues->val0,MaxIter->OptionValue@MaxIter,
             userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,Quiet->True];
                   If[True,Off[FindMinimum::sszero];Off[FindMinimum::lstol];Off[FindMinimum::bddir];
                 Off[FindMinimum::eit];Off[FindMinimum::sdir]; Off[FindMinimum::infeas];
                 Off[FindMinimum::cvmit]; Off[FindMinimum::steperr];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::cvmit]];
       If[res["ConvergenceOK"], 
          If[OptionValue@normed,AppendTo[RRL[[j]], paras /. res["NormedParas"]],AppendTo[RRL[[j]], paras /. res["sol"]]];
          AppendTo[times[[j]], t]; AppendTo[gofs[[j]], If[OptionValue@useGOFMC,First@res["GOFMC"],res["GOF1"]]];
          If[OptionValue@calcDataFit>=0,AppendTo[dfs[[j]], res["dataFit"]]]
          , 
           (*else*) fails[[j]]++;AppendTo[RRL[[j]], Table[Null,Length@paras]]];
     If[OptionValue@Verbose >= k, 
      Print[Join[{m, res["ConvergenceOK"], dRound[paras /. res["sol"]]},
                  If[OptionValue@normed,{"norm.",dRound[paras /. res["NormedParas"]]},{}],
                     {"SD L",Map[SD0,res["LLs"] ],If[OptionValue@useGOFMC,"GOFMC","GOF1"],
                        If[OptionValue@useGOFMC,First@res["GOFMC"],res["GOF1"]],
                        "GOF0",res["GOF"][[2]],"Fmin",res@"Fmin",
                       "DF",If[OptionValue@calcDataFit>=0,res["dataFit"],"" ] } ] ] ];
    ;    j = j + 1, {m, fastMeth}];
    (* slow *)
    Do[{t, res} = AbsoluteTiming@slowSEM[m, datR, obs, lat, eqs,absStat ->OptionValue@absStat,calcDataFit->OptionValue@calcDataFit,
                          userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                          LatentMean0->OptionValue@LatentMean0,InitValues->val0,MaxIter->OptionValue@MaxIter,
                          ErrorMean0->OptionValue@ErrorMean0,Quiet->True];
      If[res["ConvergenceOK"], 
          If[OptionValue@normed,AppendTo[RRL[[j]], paras /. res["NormedParas"]],AppendTo[RRL[[j]], paras /. res["sol"]]];
          AppendTo[times[[j]], t]; AppendTo[gofs[[j]], If[OptionValue@useGOFMC,First@res["GOFMC"],res["GOF1"]]];
          If[OptionValue@calcDataFit>=0,AppendTo[dfs[[j]], res["dataFit"]]], 
          (* else*) fails[[j]]++;AppendTo[RRL[[j]], Table[Null,Length@paras]]];
     If[OptionValue@Verbose >= k, 
      Print[Join[{m, res["ConvergenceOK"],dRound[ paras /. res["sol"]]},
                 If[OptionValue@normed,{"norm.",dRound[paras /. res["NormedParas"]]},{}] ,
                 {"SD L",Map[SD0,res["LLs"] ],
                  If[OptionValue@useGOFMC,"GOFMC","GOF1"],
                  If[OptionValue@useGOFMC,First@res["GOFMC"],res["GOF1"]],
                  "GOFL",res["GOFL"],"GOF0",res["GOF"][[2]],"Fmin",res@"Fmin",
                  "DF",If[OptionValue@calcDataFit>=0,res["dataFit"],"" ]   } ]]];
      j = j + 1, {m, slowMeth}],
     If[OptionValue@stopOnMessage,Print[{"StopOnMessage",k,$MessageList}];stop=True;Break[]]
    ];
     , {k, Iters}];
   RRL = N@RRL;
   result[RRR[SampleSize]] = RRL;
   result[means[SampleSize]] = Map[Map[If[Length@#>0,Mean0@#,Null]&, Transpose@#] &, RRL]; 
   result[sdss[SampleSize]] = 
    Map[Map[If[Length@#>1,SD0@#,Null]&, Transpose@#] &, RRL];
   Do[ltx = 
     ltx <> If[j==1,ToString[SampleSize],""] <> " & " <> LatexForm[paras[[j]]];
          
    Do[ ltx = 
      ltx <> " & " <> 
       ToString[
        MyRound[Map[If[Length[#]>0,Mean0[#],Null]&, 
        If[Length[Transpose@RRL[[i]]]==Length[If[OptionValue@normed,OptionValue@normedTrue,truevalues]],
             Transpose@RRL[[i]] - If[OptionValue@normed,OptionValue@normedTrue,truevalues],{}]   
        ], 
          0.1^digits][[j]]] <> If[OptionValue@ltx2Lines,"","(" <>
       ToString[Round[Map[If[Length[#]>1,SD0[#],Null]&, Transpose@RRL[[i]]], 0.1^digits][[j]]] <> ")"], {i, 1, mc}];
    ltx = ltx <> "\\\\
";
If[OptionValue@ltx2Lines, ltx=ltx<>" & "<>ToString[Round[truevalues[[j]],0.1^digits]];
Do[ ltx = 
      ltx <> " & (" <> 
       ToString[ Round[Map[If[Length[#]>1,SD0[#],Null]&, Transpose@RRL[[i]]], 0.1^digits][[j]]] <> ")", {i, 1, mc}];
    ltx = ltx <> "\\\\
";];
If[OptionValue@ltx3Lines, ltx=ltx<>" & RMSE";
Do[ rmse=Map[If[Length[#]>1,RMSE[#,truevalues[[j]]],  Null]&, Transpose@RRL[[i]]]; 
  ltx = 
      ltx <> " & " <> 
       ToString[ Round[rmse, 0.1^digits][[j]]] <> "", {i, 1, mc}];
    ltx = ltx <> "\\\\
";]

, {j, 1, Length@paras}];

  If[OptionValue@calcDataFit>=0, ltx = ltx <>  " & dfit ";
         
   Do[ ltx = 
     ltx <> " & " <> ToString[MyRound[If[Length[dfs[[i]]]>0,Mean0[dfs[[i]] ],"nc"], 0.001]] , {i,   1, mc}];
   ltx = ltx <> "\\\\
"; ];

  If[OptionValue@ltxGOF, ltx = ltx <>  " & GOF ";
         
   Do[ ltx = 
     ltx <> " & " <> ToString[MyRound[If[Length[gofs[[i]]]>0,Mean0[gofs[[i]] ],"nc"], 0.001]] , {i,   1, mc}];
   ltx = ltx <> "\\\\
"; ];


   ltx = ltx <>  " & time ";
         
   Do[ ltx = 
     ltx <> " & " <> ToString[Round[Mean[times[[i]] ], 0.1]] , {i,   1, mc}];
   ltx = ltx <> "\\\\
"; 

If[Max@fails>0,ltx = ltx <> ToString[SampleSize] <> " & fails ";
         Do[ ltx = ltx <> " & " <> ToString[fails[[i]]], {i, 1, mc}];
   ltx = ltx <> "\\\\
"];

   ltx = ltx <> "\\hline
";
   If[OptionValue@Verbose > 0 || OptionValue@printSummary, 
    Print["---------------------------------------------------- " <> 
      ToString[SampleSize]];
      rep=Join[{{"Method", paras, "std", "#fails", "avg time","avg GOF","SD GOF","avg DF","SD DF"}}, 
       Transpose@{Join[If[OptionValue@bollen, {"lavaan"}, {}], 
          fastMeth, slowMeth], 
         Map[Map[MeanRound, Transpose@#] &, RRL], 
         Map[Map[StandardDeviationRound, Transpose@#] &, RRL], fails, 
         Round[Map[Mean0, times], 0.1],Round[Map[If[Length[#]>0,Mean0[#],#]&, gofs], 0.001],
         Round[Map[If[Length[#]>1,SD0[#],#]&, gofs], 0.001],
         Round[Map[If[Length[#]>0,Mean0[#],#]&, dfs], 0.001],Round[Map[If[Length[#]>1,SD0[#],#]&, dfs], 0.001]
         } ];
    If[OptionValue@calcDataFit==-1,rep=Map[Drop[#,-2]&,rep]];     
    Print[rep // MatrixForm]
         ],
   {SampleSize, SampleSizeList} ];
  ltx = ltx <> "\\hline
\t\\end{tabular}}
\t\\label{tab:1}
\\end{table}";
  result["ltx"] = ltx;
  result["datasets"]=datasets;
  result
  ];






   (******************* SLOW - experimental XXX  ******************)




   Options[slowSEMxxx] = {MaxIter -> 500, Prec->5, Title->"",
   userLatentConstr -> {}, userConstr -> {},userOptim -> {}, LatentMean0 -> False, 
   weightPreferEqual -> False, ErrorMean0 -> -1, ErrorCor0 -> 1, 
   ErrorLatCor0 -> 1, ErrorDatCor0 -> 0, showCor -> False, 
   ErrorLatOnlyRelevant -> False, ErrorCorOnlyRelevant -> False, 
   OffDiagFactorE->1,OffDiagFactorL->1,OffDiagFactorD->1,
   weightExpo -> 1, alphaFactor -> 1,(*bringt nix*)
   LatCExpo -> 0,(*1 bringt eher nix,
   obwohl die erste schätzung besser wird*)LatCFactor -> 0, 
   EEfactor -> 1, weightCons -> 0, EFactor -> 5, F0EE -> False, doIterStep->0,
   InitValues -> {}, CorAbs -> False, alphaStart -> 1, InitValsFromSimplifiedModel->False,InitValsFromF0->False,InitValsFromF0Method->"W1",
   InitPV->1, WeightVarBound->0, (* in last step *)
   AutoInit -> True, Verbose -> False,Quiet -> False, ShowWarnings -> False, 
   calcDataFit -> -1,(*-1: no, 0=unweighted, 1 weigted, 2, weighte by mean *)
   calcDataFitWeighted -> False, WE -> -1, absStat -> False, minWeight -> 0, 
   watchParas -> {}, optIter -> 5, onlyFirstStep -> True, onlyInitStep -> False, 
   normWeight -> False, SqrtWeights0->False,SqrtWeights6->False, VarCutFactor->10, StepP->1,Itereps-> 0.0001,
   Ipfe -> 1, Ipfl -> 1, Ipfd -> 1, Iw -> 1, useNewW -> True, useNewWF->0.9,
    LastStep -> 0, WEexpo -> 1,WPenalty->10,StrategyTheta->False};


latvar=.;
slowSEMxxx[data_, observed_, latentvars_, eqs_, OptionsPattern[]] := 
  Module[{m = Length[eqs], n = Length[data], ok, i, i1, j, k, w, 
    StandardDeviation0, Variance0,minw=0.001,makeMW,t0 = AbsoluteTime[],
    F, Fe=0,FE, FD, FL,Fsol,prec=OptionValue@Prec, errmean,
    Fvars, prules = {}, 
    userConstrVals, latvarE, latvarEVal, K0,GOF1i=10^10,
    constr=OptionValue@userConstr , obsrules, EQ, Rhs, Lhs, OptionValueInitPV,
     eltable, abseps, MyAbs, MySqrt, initvalrules, AllEEs = {}, 
    GetAllVariables, getAllVariables, tdata,tdatamean, errvars = Last /@ eqs, 
    val0 = {}, freeparas, indexOf, lvv, NonNull, headlist, Vals, lv, NormV,
    res, lastsol, www, www1, neww,  
    weight, quoteData = Count[(FreeQ[#1, Null] &) /@ Flatten[data], True]/(Length[data]*Length[First[data]])}, 
    
        If[OptionValue@Quiet,Off[FindMinimum::lstol];Off[FindMinimum::bddir];Off[FindMinimum::eit];Off[FindMinimum::sdir];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::sszero];
                 Off[FindMinimum::sszero];Off[FindMinimum::cvmit];Off[FindMinimum::infeas];];
   If[OptionValue[Verbose], Print[{"Enter slowSEM Quiet=",OptionValue@Quiet}]];
      NormV[L_]:= L/Apply[Plus,L];
 makeMW[L_List]:= Module[{minw0=Mean[Abs@L]/OptionValue@VarCutFactor},(*Print[{"minwcor",L,Map[#<minw0&,L]}];*)
         Map[If[#<minw0,minw0,#]&,L]];  (* ???? *)
 
  


   NonNull[L_] := Select[L, TrueQ[FreeQ[#1, Null]] &];
   abseps = 0.0001; MyAbs[x_] := Sqrt[x^2 + abseps];MySqrt[x_] := Sqrt[x + abseps];
   tdata = Transpose@data;
   tdatamean=Map[Mean[NonNull[#]]&,tdata];
   StandardDeviation0[L_] := 
    StandardDeviation[Select[L, Not[TrueQ[Null == #]] &]];
   Variance0[L_] := Variance[Select[L, FreeQ[#, Null] &]];
   indexOf[vars_, Name_] := 
    Module[{r = 0, ii}, 
     Do[If[Name == vars[[ii]], r = ii], {ii, 1, Length[vars]}]; r];
   Vals[v_] :=    Module[{p = indexOf[observed, v], ii}, Table[data[[ii]][[p]], {ii, 1, n}]];
   lvv[v_] := Module[{ii}, Table[v[ii], {ii, 1, n}]];
   myDot[a_, b_] := 
    Module[{kk}, 
     Sum[If[FreeQ[a[[kk]], Null] && FreeQ[b[[kk]], Null], 
       a[[kk]]*b[[kk]], 0], {kk, 1, Length[a]}]];
   Correlation0[A_, B_] := 
    Module[{a = {}, b = {}, ii}, 
     If[Length[A] != Length[B], Return["Err"]];
     Do[If[TrueQ[NumberQ[A[[ii]]] && NumberQ[B[[ii]]]], 
       AppendTo[a, A[[ii]]]; AppendTo[b, B[[ii]]]], {ii, 1,  Length[A]}];
     If[Length[a] > 1, 
      If[StandardDeviation[a]*StandardDeviation[b] < 1/10^9, "Novar",  Correlation[a, b]], "NoData"]];
   Rhs[(a_) == (b_)] := b; Lhs[(a_) == (b_)] := a;
   headlist = {Or, And, Equal, Unequal, Less, LessEqual, Greater,  GreaterEqual, Inequality};
   getAllVariables[(f_)?NumericQ] := {};
   getAllVariables[{}] := {};
   getAllVariables[a_ -> b_] := 
    Union[getAllVariables[a], getAllVariables[b]];
   getAllVariables[t_] /; MemberQ[headlist, t] := {};
   getAllVariables[ll_List] :=  Union[Flatten[Union[(getAllVariables[#1] &) /@ ll]]];
   getAllVariables[Derivative[n_Integer][f_][arg__]] :=  getAllVariables[{arg}];
   getAllVariables[(f_Symbol)[arg__]] := 
    Union[{Module[{fvars}, 
       If[
        MemberQ[Attributes[f], NumericFunction] || 
         MemberQ[headlist, f], fvars = getAllVariables[{arg}], 
        fvars = f[arg]]; fvars]}]; 
   getAllVariables[other_] := {other};
   GetAllVariables[a_] := Union[Flatten[getAllVariables[a]]];

   If[OptionValue[Verbose], Print["Helper functions defined"]];


   obsrules = 
    Flatten[Table[
      Table[Rule @@ {observed[[j]][i], data[[i, j]]}, {i, 1, 
        Length[data]}], {j, 1, Length[observed]}]];
   freeparas = 
    Complement[GetAllVariables[First /@ eqs], 
     Union[observed, latentvars, (#1[[2]] &) /@ eqs]];
   val0 = Join[val0, ({#1, 0.1} &) /@ freeparas];
   Do[val0 = 
     Join[val0, Table[{lv[i], RandomReal[]/100}, {i, 1, n}]], {lv, latentvars}];
   (*latvar[lv]=Varianz latent Variable;
   latvarE Variabe für apriori schätzung latvarEVal=
   Liste der apriori schätzungen*)
   EQ[{eq_, err_}] := 
    Module[{EE, EEs, manifest,  vars, ec}, 
     EE = Lhs[eq] - Rhs[eq]; vars = GetAllVariables[EE];
     manifest = 
      Intersection[observed, Flatten[Complement[vars, latentvars]]];
     EEs = 
      Table[EE /.   
        Union[Table[
          manifest[[j]] -> 
           data[[i]][[indexOf[observed, manifest[[j]]]]], {j, 1, 
           Length[manifest]}], 
         Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1, 
           Length[latentvars]}]], {i, 1, n}];
     If[err === Null, If[OptionValue@Verbose,Print["Adding zero error constraint"]]; 
      Map[AppendTo[constr, #==0] &, EEs]; AppendTo[AllEEs, Table[0, n]], 
      AppendTo[AllEEs, EEs]];];
   EQ /@ eqs;
   If[OptionValue[Verbose], Print["EQS added"]];
   If[OptionValue[ErrorMean0] == -1, 
    constr = Join[constr, (Plus @@ NonNull[#1] == 0 &) /@ AllEEs]];
   If[OptionValue[ErrorMean0] > -1, 
    constr = Join[constr, Table[Plus @@ NonNull[AllEEs[[i]]] == n*errmean[eqs[[i,2]]] ,{i,1, Length@AllEEs}]];
    val0 = Join[val0, Table[{errmean[eqs[[i,2]]],0} ,{i,1, Length@eqs}]]];
   If[OptionValue[ErrorMean0] > 0, 
    Fe = Fe + OptionValue[ErrorMean0]*
       Sum[Plus @@ NonNull[AllEEs[[i]]]^2, {i, 1, Length[eqs]}]];
   (*Für autoinit*)constr00 = constr;
   constr = Join[constr, Table[latvar[lv]*(n - 1) == Sum[lv[i]^2, {i, 1, n}], {lv, latentvars}]];
   If[OptionValue@LatentMean0,
       Do[AppendTo[constr, 0  == Sum[latentvars[[j]][i], {i, 1, n}]], {j,1, Length@latentvars}],
       (* else: LatMean<>0*)
     constr = Join[constr, Table[latmean[j]*n  == Sum[latentvars[[j]][i], {i, 1, n}], {j,1, Length@latentvars}]];
     val0 = Join[val0, Table[{latmean[j], 0}, {j, 1, Length@latentvars}]];
       ];
  
   constr = Join[constr, Table[latvar[lv] > 0.0001, {lv, latentvars}]];
   latvarEVal = Table[1, Length@latentvars];
   val0 = Join[val0, Table[{latvar[latentvars[[j]]], latvarEVal[[j]]}, {j, 1, Length@latentvars}]];
   If[OptionValue@WeightVarBound!=0,AppendTo[val0,{K0,1}];AppendTo[constr,K0>0.0001]];
   normV[v_] := v/Apply[Plus, v];
   Do[Do[
     AppendTo[constr, 
       If[OptionValue@ErrorMean0>-1,
       EVM[j, k] == (1/n)*myDot[(AllEEs[[j]]-errmean[eqs[[j,2]]]), (AllEEs[[k]]-errmean[eqs[[k,2]]]) ],
       EVM[j, k] == (1/n)*myDot[AllEEs[[j]], AllEEs[[k]] ]
      ] ];
     If[j == k, AppendTo[constr, EVM[j, j] > 0.0000000001]];
     AppendTo[val0, {EVM[j, k], 0.001 + RandomReal[]/10}], {j, k, m}], {k, 1, m}];
 If[OptionValue[Verbose], Print["EVM added"]];

If[!OptionValue@StrategyTheta,
   Do[Do[
     AppendTo[constr, 
       If[OptionValue@ErrorMean0>-1,
         ELM[j, k] == 
           1/(n - 1)*Sum[(latentvars[[j]][i]-If[OptionValue@LatentMean0,0,latmean[j]])*If[FreeQ[AllEEs[[k, i]], Null], (AllEEs[[k, i]]-errmean[eqs[[k,2]]]  ), 0], {i, 1, n}],   
       ELM[j, k] == 
       1/(n - 1)*Sum[(latentvars[[j]][i]-If[OptionValue@LatentMean0,0,latmean[j]])*If[FreeQ[AllEEs[[k, i]], Null], AllEEs[[k, i]], 0], {i, 1, n}]   
       ]];
     AppendTo[val0, {ELM[j, k], 0.001 + RandomReal[]/100}], {j, 1, Length@latentvars}], {k, 1, m}];
 If[OptionValue[Verbose], Print["ELM added"]];
          
   Do[Do[AppendTo[constr, 
      EDM[j, k] == 1/(n - 1)*Sum[If[FreeQ[tdata[[j, i]]*AllEEs[[k, i]], Null], 
         (tdata[[j, i]]-tdatamean[[j]])*(AllEEs[[k, i]]-If[OptionValue@ErrorMean0>-1,errmean[eqs[[k,2]]],0]), 0], {i, 1, n}]];
     AppendTo[val0, {EDM[j, k], 0.001 + RandomReal[]/100}], {j, 1,Length@tdata}], {k, 1, m}];
  If[OptionValue[Verbose], Print["EDM added"]];
];  
  
   
   (* Durch die Noirmierung mit 2/(n-3)^2 sind alle summanden von FE,
   FD,FL mit Variuanz 1 *)
   FE = (n - 3)*Sum[Sum[EVM[j, k]^2/(EVM[j, j]*EVM[k, k]), {j, k + 1, m}], {k, 1,m - 1}];
   FL = (n - 3)*Sum[Sum[ELM[jj, kk]^2/(latvar[latentvars[[jj]]]*EVM[kk, kk]) , {jj, 1, Length@latentvars}], {kk, 1, m}];
   FD = (n - 3)*Sum[Sum[EDM[jj, kk]^2/((0.000001+Variance0[tdata[[jj]]])*EVM[kk, kk]) , {jj,1, Length@tdata}], {kk, 1, m}];
   
   
 F0=Sum[weight[j]^2*EVM[j, j],{j,1,m}];
   F1=F0+OptionValue[OffDiagFactorE]*FE+OptionValue[OffDiagFactorL]*FL+OptionValue[OffDiagFactorD]*FD;
 If[OptionValue[OffDiagFactorE]<0,AppendTo[constr,FE==0]];
 If[OptionValue[OffDiagFactorL]<0,AppendTo[constr,FL==0]];
 If[OptionValue[OffDiagFactorD]<0,AppendTo[constr,FD==0]];


   If[OptionValue@StrategyTheta,
      constr=Join[constr,{1==Sum[weight[j]^2*EVM[j, j],{j,1,m}]},Table[weight[j]>0.0001,{j,1,m}]],
      constr=Join[constr,{1==Product[weight[j]^2*EVM[j, j],{j,1,m}]},Table[weight[j]>0.0001,{j,1,m}]]
   ];
   val0=Join[val0,Table[{weight[j],1},{j,1,m}]];
   Fvars = GetAllVariables@{F1, constr};
   selvars = Select[val0, MemberQ[Fvars, First[#]] &];
   selvars0 = Join[selvars,  Map[{#, 0.1} &, Complement[Fvars, Map[First, selvars]]]];
   selvars = {}; 
   Do[If[FreeQ[selvars, First@p], AppendTo[selvars, p]], {p,  selvars0}];
   Do[AppendTo[val0,{v,1.0/m}],{v,Complement[Fvars,Map[First,val0]]}];
   val00=val0; val0={}; varsfound={};
   Do[If[Not[MemberQ[varsfound,val00[[j,1]]]] ,AppendTo[val0,val00[[j]]];AppendTo[varsfound,val00[[j,1]]]],{j,1,Length@val00}];
   If[OptionValue[Verbose], Print[{"Init done",val0}]];
  

Gconstr=constr;
GF=1/Product[weight[j]^2*EVM[j, j],{j,1,m}];
   If[OptionValue@StrategyTheta,
       ok = Check[sol = FindMinimum[{PowerExpand[Log[1/Product[weight[j]^2*EVM[j, j],{j,1,m}]]],constr},val0,MaxIterations->5000]; True, False],
       ok = Check[sol = FindMinimum[{F1,constr},val0,MaxIterations->5000]; True, False] 
   ]; 
   If[OptionValue[Verbose], Print[{"Optim done",sol}]];
   If[!ok,Print["Warnung Konvergenz"]];
   sol=sol[[2]];




   If[OptionValue@Verbose, Print[{"Starting report",If[ok,OptionValue@watchParas /. sol,"not ok"]}]];
   res = 
    Association["n" -> n, "data" -> data, "latentvars" -> latentvars, "Title"->OptionValue@Title,
     "eqs" -> eqs, "observed" -> observed, "ConvergenceOK"->ok,"val0"->val0,
     "paras" -> Complement[GetAllVariables[First /@ eqs], observed]];
   res["F"] = F1;
   res["constr"] = constr; 
   res["vars"] = GetAllVariables[Join[{F1}, constr]];
   res["sol"] = sol;
   
   If[!ok&&!OptionValue@Quiet,Print["Warning: slowSEM did not succeed"];Return[res]];

   res["weights"] = www;
   res["weightsN"] = res["weights"] /Apply[Plus,res["weights"]]; 
   res["weightsPost"]=1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol];
   res["weightsPostN"] = res["weightsPost"] /Apply[Plus,res["weightsPost"]]; 
   res["Fmin"] = F0 /. sol;
   res["Fmin0"] = F0 /. weight[_] :> 1/Sqrt[m] /. res["sol"];
   res["R"] = (F00 /. res["sol"])/(quoteData*n);
   res["R0"] = (F00 /. res["sol"])/(quoteData*n*m);
   res["parasol"] = Select[sol, MemberQ[res["paras"], First[#1]] &];
   res["LLs"] = (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLmeans"] = Mean /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLsds"] =  StandardDeviation /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["EEs"] = AllEEs /. sol;
   res["EVars"] = (Variance[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EEmeans"] = (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EMeans"] =  (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
    res["EE0s"] =  Table[Table[AllEEs[[j, i]], {i, 1, n}], {j, 1, Length[errvars]}] /.  sol;
   res["LEcors"] = Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], res["LLs"][[j, s]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
         {i, 1, m}, {j, 1, Length@latentvars}];
   res["DEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] && NumberQ[data[[s,j]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], data[[s,j]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, Length@observed}];
   res["EEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1,  n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, m}];
   res["EEcovs"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[
         dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1, 
        n}]; Covariance @@ Transpose[dat]], {i, 1, m}, {j, 1, m}];

    res["GOFMC"]=GOFMC[res];

    res["GOF"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Mean@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Mean@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
              (* "weightPostweightAgreement",Norm[res["weightsPost"]/Apply[Plus,res["weightsPost"]]-
                                                res["weights"]/Apply[Plus,res["weights"]] ], *)
                "SDWeightErr",StandardDeviation[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]};     
  res["GOF1"]=Check[Mean[{m,1,1,1,0}*Select[res["GOF"],NumberQ]],Null];
  If[GOF1i*1.5<res["GOF1"],res["ConvergenceOK"]=False;If[!OptionValue@Quiet,Print[{"GOF1 convergence warning",GOF1i,res["GOF1"],{b1,b2,b3}/.res["sol"]}]]];
res["GOFL"]=Map[Mean,GOFList2/.res["sol"]];
   res["GOFM"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Max@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Max@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
               "SDWeightErr",StandardDeviation[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]};     
  res["GOF2"]=Check[Mean[{m,1,1,1,0}*Select[res["GOFM"],NumberQ]],Null];


 res["NormedParas"]=calcNormedParas[res];


  If[OptionValue@calcDataFit>=0, 
      res["dataFit"]=CalcDataFit[res,OptionValue@calcDataFit>0,OptionValue@calcDataFit],
      res["dataFit"]="Data fit not calculated" ] ;



   res["eqssol"] = Table[{eqs[[i, 1]] /. res["parasol"], res["EVars"][[i]]}, {i, 1,  Length@eqs}];
   res["eqssolNorm1"] = 
    res["eqssol"] /.  
     Table[Apply[
       Rule, {latentvars[[i]], 
        latentvars[[i]]*StandardDeviation[res["LLs"][[i]]]}], {i, 1,  Length@latentvars}];
   res["eqssolNorm1b"] = res["eqssol"];
   powersOfDeg[vars_List, n_] := 
    If[n == 1, vars, 
     Module[{p, v, P = powersOfDeg[vars, n - 1]}, Flatten[Table[p*v, {p, P}, {v, vars}]]]];
   Module[{p, vp, np}, 
    Do[Do[np = 
       ToExpression@
        StringReplace[
         ToString[p, InputForm], {"*" -> "M", "^" -> "P"}];
      AppendTo[prules, Apply[Rule, {np, p}]];
      vp =  StandardDeviation[
        p /.  Map[Apply[Rule, #] &, 
          Map[{#, res["LLs"][[Position[latentvars, #][[1, 1]]]]} &,  latentvars]]];
      res["eqssolNorm1b"] = 
       res["eqssolNorm1b"] /. Apply[Rule, {p, vp*np}], {p, 
       powersOfDeg[latentvars, deg]}], {deg, {3, 2, 1}}]];
   NormEQ2[a_ == b_] := 
    Module[{ip = Position[observed, a][[1, 1]], 
       dataT = Transpose@data}, 
      a == Expand[b/N[StandardDeviation0[dataT[[ip]]]]]] /;   MemberQ[observed, a];
   NormEQ2[a_ == b_] := 
    Module[{ip = Position[latentvars, a][[1, 1]]}, 
      a == Expand[b/N[StandardDeviation0[res["LLs"][[ip]]]]]] /;  MemberQ[latentvars, a];
   res["eqssolNorm2"] =  Map[{NormEQ2[#[[1]]], #[[2]]} &, res["eqssolNorm1"]];
   res["eqssolNorm2b"] = Map[{NormEQ2[#[[1]]], #[[2]]} &, res["eqssolNorm1b"] /. prules];
    res["time"]= AbsoluteTime[]-t0;
    If[OptionValue@Quiet,On[FindMinimum::lstol];On[FindMinimum::bddir];On[FindMinimum::eit];On[FindMinimum::sdir];
                 On[FindMinimum::cvmit];On[FindMinimum::acceptlev];On[FindMinimum::lstol];On[FindMinimum::cvmit];
                 On[FindMinimum::sszero];On[FindMinimum::infeas];On[FindMinimum::sszero];];
    res["eqssolNorm"] = 
    Table[{eqs[[i, 1]] /. res["NormedParas"], res["EVars"][[i]]}, {i, 1, Length@eqs}];
   

   res];



(******************* SLOW2  - experimentell   ******************)




   Options[slowSEM2] = {MaxIter -> 500, Prec->5, Title->"",avgvor->True, dosol2->False,dosol3->0,
  ELDFactorPass1->0.1,ELDFactorPass2->1,avgvorSqrt->False,
   userLatentConstr -> {}, userConstr -> {},userOptim -> {},   
   weightPreferEqual -> False, ErrorMean0 -> -1, ErrorCor0 -> 1, 
   ErrorLatCor0 -> 1, ErrorDatCor0 -> 0, showCor -> False, LatentMean0 -> False,
   ErrorLatOnlyRelevant -> False, ErrorCorOnlyRelevant -> False, 
   weightExpo -> 1, alphaFactor -> 1,(*bringt nix*)
   LatCExpo -> 0,(*1 bringt eher nix,
   obwohl die erste schätzung besser wird*)LatCFactor -> 0, 
   EEfactor -> 1, weightCons -> 0, EFactor -> 5, F0EE -> False, doIterStep->0,
   InitValues -> {}, CorAbs -> False, alphaStart -> 1, InitValsFromSimplifiedModel->False,InitValsFromF0->False,InitValsFromF0Method->"W1",
   InitPV->1, WeightVarBound->0, (* in last step *)
   AutoInit -> True, Verbose -> False,Quiet -> False, ShowWarnings -> False, 
   calcDataFit -> -1,(*-1: no, 0=unweighted, 1 weigted, 2, weighte by mean *)
   calcDataFitWeighted -> False, WE -> -1, absStat -> False, minWeight -> 0, 
   watchParas -> {}, optIter -> 5, onlyFirstStep -> True, onlyInitStep -> False, 
   normWeight -> False, SqrtWeights0->False,SqrtWeights6->False, VarCutFactor->10, StepP->1,Itereps-> 0.0001,
   Ipfe -> 1, Ipfl -> 1, Ipfd -> 1, Iw -> 1, useNewW -> True, useNewWF->0.9,
    LastStep -> 0, WEexpo -> 1,WPenalty->10};



(***************** Experimentell: Latente Variablen nicht normiert *********** *)


slowSEM2["flex",data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=Module[{res1,res2,res3,res4},
   res1=Apply[slowSEM2  ,{ "Normal",data, observed, latentvars0, eqs0,InitValsFromF0->OptionValue@InitValsFromF0,Quiet->OptionValue@Quiet,Title->"flex",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      InitValues->OptionValue@InitValues,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
   res2=Apply[slowSEM2,{"Normal2b",data, observed, latentvars0,eqs0,Title->"flex",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
   res3=Apply[slowSEM2,{"Normal2",data, observed, latentvars0, eqs0,InitValsFromF0->OptionValue@InitValsFromF0,
                        Quiet->OptionValue@Quiet,Title->"flex",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
res4=Apply[slowSEM2,{"Normal1",data, observed, latentvars0, eqs0,InitValsFromF0->OptionValue@InitValsFromF0,
                        Quiet->OptionValue@Quiet,Title->"flex",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
    res1=If[Apply[Plus,res1["GOFL"]]<Apply[Plus,res2["GOFL"]],res1,res2] ;
    res3=If[Apply[Plus,res3["GOFL"]]<Apply[Plus,res4["GOFL"]],res3,res4] ;
    If[Apply[Plus,res1["GOFL"]]<Apply[Plus,res3["GOFL"]],res1,res3] 
    ];

slowSEM2["flex2",data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=Module[{res0,res1},
   res1=Apply[slowSEM2  ,{ "flex",data, observed, latentvars0, eqs0,InitValsFromF0->OptionValue@InitValsFromF0,Quiet->OptionValue@Quiet,Title->"flex2",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      InitValues->OptionValue@InitValues,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
   res2=Apply[slowSEM,{"W^{2a00}",data, observed, latentvars0, eqs0,InitValsFromF0->OptionValue@InitValsFromF0,
                        Quiet->OptionValue@Quiet,Title->"flex2",calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,
                      WPenalty->OptionValue@WPenalty}];
    If[Apply[Plus,res1["GOFL"]]<Apply[Plus,res2["GOFL"]],res1,res2] ];



slowSEM2[m_String,data_, observed_, latentvars0_, eqs0_, OptionsPattern[]]:=
  Apply[slowSEM2,Join[{data, observed, latentvars0, eqs0,Quiet->True,Title->m,calcDataFit->OptionValue@calcDataFit,LatentMean0 -> OptionValue@LatentMean0,
                      InitValues->OptionValue@InitValues,
                      userLatentConstr -> OptionValue@userLatentConstr, userConstr -> OptionValue@userConstr,doIterStep->OptionValue@doIterStep,
                      absStat ->OptionValue@absStat,ErrorMean0->OptionValue@ErrorMean0,Verbose->OptionValue@Verbose,Itereps-> OptionValue@Itereps,
                      WPenalty->OptionValue@WPenalty},
     Switch[m,
            "Normal",{avgvor -> True},
            "Normal1",{avgvor -> False},
            "Normal2",{avgvor -> False,dosol2->True},
            "Normal2b",{avgvor -> False,dosol2->True,ELDFactorPass2->5},
            "Normal2c",{avgvor -> False,dosol2->True,ELDFactorPass2->1},
            "Normal3",{avgvor -> True,dosol2->True}, 
            "Normal3b",{avgvor -> True,avgvorSqrt -> True,dosol2->True}, (* Nicht besser als 3 *)
            "Normal4",{avgvor -> False,dosol2->True,dosol3->0.5},
            "Normal5",{avgvor -> False,dosol2->True,dosol3->0.8},
            "Normal6",{avgvor -> False,dosol2->True,dosol3->0.9},
            "Normal7",{avgvor -> False,dosol2->True,dosol3->0.5,ELDFactorPass1->1}, 
            "W^a(1)",{useNewW->False},
            "W^a(W1)",{InitValsFromF0 -> True},
            "W^a(Wn)",{InitValsFromF0 -> True,InitValsFromF0Method->"Wn"},
            "W^a(Ww)",{InitValsFromF0 -> True,InitValsFromF0Method->"Ww(Wn)"},
            "W^a(W1a)",{InitValsFromF0 -> True,useNewW->True,useNewWF->0.5},
            "W^{2a0}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 0, useNewWF -> 0.5},
            "W^{2a0.1}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 0.1, useNewWF -> 0.5},
            "W^{2a}",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->6},
            "W^{2b}",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->6,StepP->0.0,WeightVarBound -> 0},
            "W^{2a1}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 1, useNewWF -> 0.5},
            "W^{2a10}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 10, useNewWF -> 0.5},
            "W^{2a100}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 100, useNewWF -> 0.5},
            "W^{2a1000}",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 5, onlyFirstStep -> True, WeightVarBound -> 1000, useNewWF -> 0.5},
            "W^{2a1006}",(*bad!*){InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 6, onlyFirstStep -> True, WeightVarBound -> 100, useNewWF -> 0.5},
            "I5",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True, useNewWF -> 0.5,doIterStep -> 5, StepP ->  0.001,Itereps-> 0.000001},
            "I20",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True, useNewWF -> 0.5,doIterStep -> 20, StepP -> 0.001,Itereps-> 0.000001},
            "I20b",{InitValsFromF0 -> True, onlyInitStep -> False, LastStep -> 0, onlyFirstStep -> True, useNewWF -> 0.5,doIterStep -> 20, StepP -> 0.01,Itereps-> 0.000001},
            "W^7",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->100},
            "W^7a",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->0.01},
            "W^7ab",{useNewW->True,useNewWF->0.9,InitValsFromF0->True, LastStep->7,StepP->0.0001},
            "W^7b",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->0.0}, (* good *)
            "W^7b100",{useNewW->True,useNewWF->0.5,InitValsFromF0->True, LastStep->7,StepP->0.0,WPenalty->100}, 
            _,{}
     ]
  ]];


latvar=.; latmean=.; 
slowSEM2[data_, observed_, latentvars_, eqs_, OptionsPattern[]] := 
  Module[{m = Length[eqs], n = Length[data], ok, i, i1, j, k, w, 
    StandardDeviation0, Variance0,minw=0.001,makeMW,t0 = AbsoluteTime[],
    F, Fe=0,FE, FD, FL,Fsol,FsolDebug,prec=OptionValue@Prec, errmean,
    Fvars, prules = {}, 
     latvarE, latvarEVal, K0,GOF1i=10^10,
    constr , obsrules, EQ, Rhs, Lhs, OptionValueInitPV,
     eltable, abseps, MyAbs, MySqrt, initvalrules, AllEEs = {}, 
    GetAllVariables, getAllVariables, tdata,tdatamean, errvars = Last /@ eqs, 
    val0 = {}, freeparas, indexOf, lvv, NonNull, headlist, Vals, lv, NormV,
    res, lastsol, www, www1, neww,  lat0rules, 
    weight, quoteData = Count[(FreeQ[#1, Null] &) /@ Flatten[data], True]/(Length[data]*Length[First[data]]),FEList,FLList,FDList,GOFList}, 
    
        If[OptionValue@Quiet,Off[FindMinimum::lstol];Off[FindMinimum::bddir];Off[FindMinimum::eit];Off[FindMinimum::sdir];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::sszero];
                 Off[FindMinimum::sszero];Off[FindMinimum::cvmit];Off[FindMinimum::infeas];];
   If[OptionValue[Verbose], Print[{"Enter slowSEM Quiet=",OptionValue@Quiet}]];
      NormV[L_]:= L/Apply[Plus,L];
 makeMW[L_List]:= Module[{minw0=Mean[Abs@L]/OptionValue@VarCutFactor},(*Print[{"minwcor",L,Map[#<minw0&,L]}];*)
         Map[If[#<minw0,minw0,#]&,L]];  (* ???? *)
 


   constr=OptionValue@userConstr;

   NonNull[L_] := Select[L, TrueQ[FreeQ[#1, Null]] &];
   abseps = 0.0001; MyAbs[x_] := Sqrt[x^2 + abseps];MySqrt[x_] := Sqrt[x + abseps];
   tdata = Transpose@data;
   tdatamean=Map[Mean[NonNull[#]]&,tdata];
   StandardDeviation0[L_] := 
    StandardDeviation[Select[L, Not[TrueQ[Null == #]] &]];
   Variance0[L_] := Variance[Select[L, FreeQ[#, Null] &]];
   indexOf[vars_, Name_] := 
    Module[{r = 0, ii}, 
     Do[If[Name == vars[[ii]], r = ii], {ii, 1, Length[vars]}]; r];
   Vals[v_] :=    Module[{p = indexOf[observed, v], ii}, Table[data[[ii]][[p]], {ii, 1, n}]];
   lvv[v_] := Module[{ii}, Table[v[ii], {ii, 1, n}]];
   myDot[a_, b_] := 
    Module[{kk}, 
     Sum[If[FreeQ[a[[kk]], Null] && FreeQ[b[[kk]], Null], 
       a[[kk]]*b[[kk]], 0], {kk, 1, Length[a]}]];
   Correlation0[A_, B_] := 
    Module[{a = {}, b = {}, ii}, 
     If[Length[A] != Length[B], Return["Err"]];
     Do[If[TrueQ[NumberQ[A[[ii]]] && NumberQ[B[[ii]]]], 
       AppendTo[a, A[[ii]]]; AppendTo[b, B[[ii]]]], {ii, 1,  Length[A]}];
     If[Length[a] > 1, 
      If[StandardDeviation[a]*StandardDeviation[b] < 1/10^9, "Novar",  Correlation[a, b]], "NoData"]];
   Rhs[(a_) == (b_)] := b; Lhs[(a_) == (b_)] := a;
   headlist = {Or, And, Equal, Unequal, Less, LessEqual, Greater,  GreaterEqual, Inequality};
   getAllVariables[(f_)?NumericQ] := {};
   getAllVariables[{}] := {};
   getAllVariables[a_ -> b_] := 
    Union[getAllVariables[a], getAllVariables[b]];
   getAllVariables[t_] /; MemberQ[headlist, t] := {};
   getAllVariables[ll_List] :=  Union[Flatten[Union[(getAllVariables[#1] &) /@ ll]]];
   getAllVariables[Derivative[n_Integer][f_][arg__]] :=  getAllVariables[{arg}];
   getAllVariables[(f_Symbol)[arg__]] := 
    Union[{Module[{fvars}, 
       If[
        MemberQ[Attributes[f], NumericFunction] || 
         MemberQ[headlist, f], fvars = getAllVariables[{arg}], 
        fvars = f[arg]]; fvars]}]; 
   getAllVariables[other_] := {other};
   GetAllVariables[a_] := Union[Flatten[getAllVariables[a]]];

   If[OptionValue[Verbose], Print["Helper functions defined"]];


   obsrules = 
    Flatten[Table[
      Table[Rule @@ {observed[[j]][i], data[[i, j]]}, {i, 1, 
        Length[data]}], {j, 1, Length[observed]}]];
   freeparas = 
    Complement[GetAllVariables[First /@ eqs], 
     Union[observed, latentvars, (#1[[2]] &) /@ eqs]];
   val0 = Join[val0, ({#1, 0.1} &) /@ freeparas];
   Do[val0 = Join[val0, Table[{lv[i], RandomReal[]/100}, {i, 1, n}]], {lv, latentvars}];
   (*latvar[lv]=Varianz latent Variable;
   latvarE Variabe für apriori schätzung latvarEVal=
   Liste der apriori schätzungen*)
   EQ[{eq_, err_}] := 
    Module[{EE, EEs, manifest,  vars, ec}, 
     EE = Lhs[eq] - Rhs[eq]; vars = GetAllVariables[EE];
     manifest = 
      Intersection[observed, Flatten[Complement[vars, latentvars]]];
     EEs = 
      Table[EE /.   
        Union[Table[
          manifest[[j]] -> 
           data[[i]][[indexOf[observed, manifest[[j]]]]], {j, 1, 
           Length[manifest]}], 
         Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1, 
           Length[latentvars]}]], {i, 1, n}];
     If[err === Null||err===0, If[OptionValue@Verbose,Print["Adding zero error constraint"]]; 
      Map[AppendTo[constr, #==0] &, EEs]; AppendTo[AllEEs, Table[0, n]], 
      AppendTo[AllEEs, EEs]];];
   EQ /@ eqs;
   If[OptionValue[Verbose], Print["EQS added"]];
   If[OptionValue[ErrorMean0] == -1, 
    constr = Join[constr, (Plus @@ NonNull[#1] == 0 &) /@ AllEEs]];
   If[OptionValue[ErrorMean0] > -1, 
    constr = Join[constr, Table[Plus @@ NonNull[AllEEs[[i]]] == n*errmean[eqs[[i,2]]] ,{i,1, Length@AllEEs}]];
    val0 = Join[val0, Table[{errmean[eqs[[i,2]]],0} ,{i,1, Length@eqs}]]];
   If[OptionValue[ErrorMean0] > 0, 
    Fe = Fe + OptionValue[ErrorMean0]*
       Sum[Plus @@ NonNull[AllEEs[[i]]]^2, {i, 1, Length[eqs]}]];
   (*Für autoinit*)constr00 = constr;
   constr = Join[constr, Table[latmean[lv]*n == Sum[lv[i], {i, 1, n}], {lv, latentvars}]];
   If[OptionValue@LatentMean0,constr = Join[constr, Table[0 == Sum[lv[i], {i, 1, n}], {lv, latentvars}]]];
   constr = Join[constr, Table[latvar[lv]*(n - 1) == Sum[(lv[i]-latmean[lv])^2, {i, 1, n}], {lv, latentvars}]];
   constr = Join[constr, Table[latvar[lv] > 0.0001, {lv, latentvars}]];
   latvarEVal = Table[1, Length@latentvars];
   val0 = Join[val0, Table[{latvar[latentvars[[j]]], latvarEVal[[j]]}, {j, 1, Length@latentvars}]];
   val0 = Join[val0, Table[{latmean[latentvars[[j]]], 0}, {j, 1, Length@latentvars}]];
   If[OptionValue@WeightVarBound!=0,AppendTo[val0,{K0,1}];AppendTo[constr,K0>0.0001]];
   normV[v_] := v/Apply[Plus, v];
   constr = 
    Join[constr, 
     Select[Flatten[
       Table[OptionValue@userLatentConstr /. 
         Join[Table[latentvars[[j]] -> latentvars[[j]][i], {j, 1,  Length[latentvars]}],                                 
              Table[eqs[[j, 2]] -> AllEEs[[j, i]], {j, 1, m}]], {i, 1,  n}]], FreeQ[#, Null] &]];
   Do[Do[
     AppendTo[constr, 
       If[OptionValue@ErrorMean0>-1,
       EVM[j, k] == (1/n)*myDot[(AllEEs[[j]]-errmean[eqs[[j,2]]]), (AllEEs[[k]]-errmean[eqs[[k,2]]]) ],
       EVM[j, k] == (1/n)*myDot[AllEEs[[j]], AllEEs[[k]] ]
      ] ];
     If[j == k, AppendTo[constr, EVM[j, j] > 0.0000000001]];
     AppendTo[val0, {EVM[j, k], 0.001 + RandomReal[]/10}], {j, k, m}], {k, 1, m}];
 If[OptionValue[Verbose], Print["EVM added"]];

   Do[Do[
     AppendTo[constr, 
       If[OptionValue@ErrorMean0>-1,
         ELM[j, k] == 
           1/(n - 1)*Sum[(latentvars[[j]][i]-latmean[latentvars[[j]]])*If[FreeQ[AllEEs[[k, i]], Null], (AllEEs[[k, i]]-errmean[eqs[[k,2]]]  ), 0], {i, 1, n}],   
       ELM[j, k] == 
       1/(n - 1)*Sum[(latentvars[[j]][i]-latmean[latentvars[[j]]])*If[FreeQ[AllEEs[[k, i]], Null], AllEEs[[k, i]], 0], {i, 1, n}]   
       ]];
     AppendTo[val0, {ELM[j, k], 0.001 + RandomReal[]/100}], {j, 1, Length@latentvars}], {k, 1, m}];
 If[OptionValue[Verbose], Print["ELM added"]];
          
   Do[Do[AppendTo[constr, 
      EDM[j, k] == 1/(n - 1)*Sum[If[FreeQ[tdata[[j, i]]*AllEEs[[k, i]], Null], 
         (tdata[[j, i]]-tdatamean[[j]])*(AllEEs[[k, i]]-If[OptionValue@ErrorMean0>-1,errmean[eqs[[k,2]]],0]), 0], {i, 1, n}]];
     AppendTo[val0, {EDM[j, k], 0.001 + RandomReal[]/100}], {j, 1,Length@tdata}], {k, 1, m}];
  If[OptionValue[Verbose], Print["EDM added"]];
  
  
   
   (* Durch die Noirmierung mit 2/(n-3)^2 sind alle summanden von FE,
   FD,FL mit Variuanz 1 *)

   FE =If[OptionValue@avgvor,If[OptionValue@avgvorSqrt,Sqrt[1/(m*(m-1)/2)],1/(m*(m-1)/2)],1] *
     (n - 3)*Sum[Sum[EVM[j, k]^2/(EVM[j, j]*EVM[k, k]), {j, k + 1, m}], {k, 1,m - 1}];
   FL = If[OptionValue@avgvor,If[OptionValue@avgvorSqrt,Sqrt[1/(m*Length@latentvars)],1/(m*Length@latentvars)],1]*
       (n - 3)*Sum[Sum[ELM[jj, kk]^2/(latvar[latentvars[[jj]]]*EVM[kk, kk]) , {jj, 1, Length@latentvars}], {kk, 1, m}];
   FD = If[OptionValue@avgvor,If[OptionValue@avgvorSqrt,Sqrt[1/(m*Length@tdata)],1/(m*Length@tdata)],1]*
      (n - 3)*Sum[Sum[EDM[jj, kk]^2/((0.000001+Variance0[tdata[[jj]]])*EVM[kk, kk]) , {jj,1, Length@tdata}], {kk, 1, m}];

   FEList =(n - 3)*2*Table[Table[EVM[j, k]^2/(EVM[j, j]*EVM[k, k]), {j, k + 1, m}], {k, 1,m - 1}];
   FLList =(n - 3)*Table[Table[ELM[jj, kk]^2/(latvar[latentvars[[jj]]]*EVM[kk, kk]) , {jj, 1, Length@latentvars}], {kk, 1, m}];
   FDList =(n - 3)*Table[Table[EDM[jj, kk]^2/((0.000001+Variance0[tdata[[jj]]])*EVM[kk, kk]) , {jj,1, Length@tdata}], {kk, 1, m}];
   GOFList=Flatten[{FEList,FLList,FDList,m^2*Table[EVM[k, k], {k, 1, m}]}];
GOFList2={Flatten[{FEList,FLList,FDList}],Flatten[m^2*Table[EVM[k, k], {k, 1, m}]]};

   GFE = FE; GFL = FL; GFD = FD; 
   
   F[w_List, pfe_, pfl_, pfd_] := (F0 = Fe+Sum[w[[k]]*If[OptionValue@absStat,MySqrt@EVM[k, k],EVM[k, k]], {k, 1, m}];  
                                   F0 + pfe*FE + pfl*FL + pfd*FD+Sum[(OptionValue@userOptim)[[j]]^2 ,{j,1,Length@OptionValue@userOptim}]);
   Fsol[w_List, pfe_, pfl_, pfd_, val0_] := (F1=F[w, pfe, pfl, pfd]; 
                  (*If[Not[OptionValue@Quiet],Print[{"evm nach sol00",Table[EVM[k, k], {k, 1, m}]/.sol00}]];
                  If[Not[OptionValue@Quiet],Print[{"weights sol0",1/Mean[Table[EVM[k, k], {k, 1, m}]/.sol00],"b1b2b3 sol00",{b1,b2,b3}/.sol00}]];
                  *)
                  ok = Check[so0=FindMinimum[{Sum[EVM[k, k], {k, 1, m}]+OptionValue@ELDFactorPass1*(FE + FL + FD),constr}, val0]; True, False];
                  If[ok,weight0=1/Mean[Table[EVM[k, k], {k, 1, m}]/.so0[[2]]],wegiht0=1];
                  If[Not[OptionValue@Quiet],Print[{"weights so0",weight0,"b1,b2,b3 aus so0",{b1,b2,b3}/.so0[[2]]}] ];
                  fsol=FindMinimum[{F[
                    If[False,w,Table[weight0,{i,1,m}]], OptionValue@ELDFactorPass2*pfe, OptionValue@ELDFactorPass2*pfl, OptionValue@ELDFactorPass2*pfd], 
                    constr}, If[ok,Map[{#[[1]],#[[2]]}&,so0[[2]]],val0], MaxIterations->OptionValue@MaxIter][[2]];
                  If[OptionValue@dosol2,
                    weight0=1/Mean[Table[EVM[k, k], {k, 1, m}]/.fsol];
                    ok = Check[fsol=FindMinimum[{F[
                      If[False,w,Table[weight0,{i,1,m}]], pfe, pfl, pfd], constr}, If[ok,Map[{#[[1]],#[[2]]}&,fsol],val0], MaxIterations->OptionValue@MaxIter][[2]]; True, False];
                  ];  
                  If[OptionValue@dosol3>0,
                    weightS0=(1-OptionValue@dosol3)*Table[weight0,{k,1,m}]+OptionValue@dosol3*Table[1/EVM[k, k]/.fsol, {k, 1, m}];
                    ok = Check[fsol=FindMinimum[{F[
                      If[False,w,weightS0], pfe, pfl, pfd], constr},  If[ok,Map[{#[[1]],#[[2]]}&,fsol],val0], MaxIterations->OptionValue@MaxIter][[2]];True,False] ];  
                  fsol );


   FsolDebug[w_List, pfe_, pfl_, pfd_, val0_] := (F1=F[w, pfe, pfl, pfd];
                  If[OptionValue@Verbose,
                      Print["GFindMinimumCall asigned"];
                      GFindMinimumCall= FindMinimum0[{F[w, pfe, pfl, pfd], constr}, val0];];
                  FindMinimum[{F[w, pfe, pfl, pfd], constr}, val0, MaxIterations->OptionValue@MaxIter][[2]]);
  
  
    If[OptionValue@Iw >= 0, www = Table[OptionValue@Iw, m], 
    www = Table[(1 + m)/(1 + Length@latentvars), m]]; (* Iw<0: Heuristic that more equations per latent should reduce error - little effect *)
   pfe0 = OptionValue@Ipfe; pfl0 = OptionValue@Ipfl; 
   pfd0 = OptionValue@Ipfd; 

 If[OptionValue@InitValsFromSimplifiedModel,
    If[OptionValue@Verbose,Print["ENTERING recursion for InitValsFromSimplifiedModel"]];
    sol=slowSEM[data, observed, latentvars,  Select[eqs, (1 == Length@Intersection[latentvars, GetAllVariables[#]]) &], 
       onlyInitStep -> True, LastStep -> 0, onlyFirstStep -> True,InitValsFromSimplifiedModel ->False];
    If[!sol["ConvergenceOK"]&&!OptionValue@Quiet,Print["InitValsFromSimplifiedModel failed"]];   
    If[sol["ConvergenceOK"],sol=sol["sol"];
      Do[  If[NumberQ[First[val0[[j]]]/.sol],  val0[[j,2]]=First[val0[[j]]]/.sol   ],{j,1,Length[val0]}]
    ];
    If[OptionValue@Verbose,Print["LEAVING recursion for InitValsFromSimplifiedModel"]];
 
    ];   

    OptionValueInitPV=OptionValue@InitPV;

If[OptionValue@InitValsFromF0,
    If[OptionValue@Verbose,Print["ENTERING recursion to fastSEM for InitValsFromF0"]];
   sol0=fastSEM[OptionValue@InitValsFromF0Method,data, observed, latentvars, eqs, LatentMean0 -> True,userLatentConstr->OptionValue@userLatentConstr ,
      userConstr->OptionValue@userConstr,Quiet->OptionValue@Quiet];
            If[OptionValue@Quiet,Off[FindMinimum::sszero];Off[FindMinimum::lstol];Off[FindMinimum::bddir];
                 Off[FindMinimum::eit];Off[FindMinimum::sdir]; Off[FindMinimum::infeas];
                 Off[FindMinimum::cvmit]; Off[FindMinimum::steperr];
                 Off[FindMinimum::cvmit];Off[FindMinimum::acceptlev];Off[FindMinimum::lstol];Off[FindMinimum::cvmit]]; 
    If[OptionValue@Verbose,Print["RETURNED from recursion to fastSEM for InitValsFromF0"]];
    If[!sol0["ConvergenceOK"]&&!OptionValue@Quiet,Print["InitValsFromF0 failed"]];   
    If[sol0["ConvergenceOK"],
      OptionValueInitPV=(sol0["F"] /. sol0["sol"]);  GOF1i=sol0["GOF1"];
      sol=sol0["sol"];
      If[OptionValue@Verbose, Print[{"val0 determined", Take[sol,Min[5,Length@sol]],OptionValue@watchParas /. sol}]];
      If[Head[sol]==List&&Union[Map[Head,sol]]=={Rule},
         Do[  If[NumberQ[First[val0[[j]]]/.sol],  val0[[j,2]]=First[val0[[j]]]/.sol   ],{j,1,Length[val0]}],
         If[True||!OptionValue@Quiet,Print["solution was not rule list"]]
      ]
    ];
 ];
If[OptionValue@Verbose, Print[{"val0 determined and initialised,val0"}]];

   
   Fvars = GetAllVariables@{F[www, pfe0, pfl0, pfd0], constr};
   selvars = Select[val0, MemberQ[Fvars, First[#]] &];
   selvars0 = Join[selvars,  Map[{#, 0.1} &, Complement[Fvars, Map[First, selvars]]]];
   selvars = {}; 
   Do[If[FreeQ[selvars, First@p], AppendTo[selvars, p]], {p,  selvars0}];
   GF = F[www, pfe0, pfl0, pfd0]; Gval0 = val0; GC = constr; GFsol=Fsol; Gwww=www; 

  Do[AppendTo[val0,{v,0}],{v,Complement[Fvars,Map[First,val0]]}];

  If[OptionValue@Verbose, 
    Print[{"Now 00"}]; Print[{"Fsol arguments:",www, OptionValueInitPV, OptionValueInitPV, OptionValueInitPV, 
       Take[val0,Min[5,Length@val0]]}]];


   ok = Check[sol00 = FsolDebug[www, OptionValueInitPV, OptionValueInitPV, OptionValueInitPV, val0]; True, False]; 
   If[!ok,ok = Check[sol00 = FsolDebug[www, OptionValueInitPV, OptionValueInitPV, OptionValueInitPV,
                                   Map[{#[[1]],#[[2]]+RandomReal[{0,0.1}]}&,val0]]; True, False]]; 
  If[OptionValue@Verbose, Print["sol00 determined"]];
   
   ok=ok&&TrueQ[Head[sol00]==List&&Union[Map[Head,sol00]]=={Rule}];

   If[OptionValue@Verbose, If[ok, Print[{"sol of init step found", OptionValue@watchParas /. sol00}],
   Print["Issues in 00"] ] ];
   If[ok, val0 = Map[{#[[1]], #[[2]]} &, sol00]];
   If[!ok&&!OptionValue@Quiet,Print["Conv issues in step 00"]];
   
   If[OptionValue@onlyInitStep,sol=sol00,

   If[ok,If[OptionValue@useNewW,  
       www =(OptionValue@useNewWF)*Table[1/Mean[Table[EVM[k, k], {k, 1, m}]/.sol00] , m];
       www=www+(1-OptionValue@useNewWF)*(1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol00]) ]]; (* ususally almost no improvement *)
   
   If[OptionValue@SqrtWeights0,www=Sqrt[www]];

   
   If[OptionValue@Verbose, Print[{"Init done, status,watchParas, weights",ok,If[ok,OptionValue@watchParas/.sol00,{}], www}]];

   ok = Check[sol = Fsol[www, pfe0, pfl0, pfd0, val0]; True, False];
   If[! ok, 
    val00 = Map[{First@#, Last[#] + RandomReal[{0.05, 0.01}]} &, val0];
    ok =  Check[sol = Fsol[www, pfe0, pfl0, pfd0, val00]; True, False];
    ];
   If[! ok, 
    val00 = Map[{First@#, Last[#] + RandomReal[{0.05, 0.01}]} &, val0];
    prec=1+Floor[OptionValue@Prec/2];
    ok =  Check[sol = Fsol[www, pfe0/10, pfl0/10, pfd0/10, val00]; True, False];
    prec=OptionValue@Prec
    ];
    ok=ok&&TrueQ[Head[sol]==List&&Union[Map[Head,sol]]=={Rule}];
   If[! ok &&!OptionValue@Quiet, Print[{"Conv issues in first step"}]];
   If[OptionValue@Verbose, Print[{ok,"First sol=", If[ok,OptionValue@watchParas /. sol,"not ok"]}]];
   ]; (* end of else-part of  of If[OptionValue@onlyInitStep*)

   If[Not[OptionValue@onlyFirstStep]&&Not[OptionValue@onlyInitStep],
    
    www1 = If[ok,1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol],www]; 
    If[OptionValue@Verbose, Print[{"SecondStep, weights",www1}]];
    If[OptionValue@normWeight, www1 = normV@www1];
    ok = Check[sol1 = Fsol[www1, pfe0, pfl0, pfd0, If[ok,Map[{First@#, Last@#} &, sol],val0]]; True, False];
    If[! ok, 
     val00 = Map[{First@#, Last[#] + RandomReal[{0.05, 0.01}]} &, val0];
     ok =  Check[sol1 =  Fsol[If[ok,1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol,www1]], pfe0, pfl0, pfd0,  val00]; True, False];
     ];
    If[! ok, If[!OptionValue@Quiet,Print[{"Conv issues in step 1"}]], (* else step ok *) sol = sol1];  
    ]; (* End only 1st step *) 

    If[OptionValue@Verbose&&Not[OptionValue@onlyFirstStep], 
          Print[{"Now optional steps",ok,If[ok,OptionValue@watchParas/.sol,"not ok"]}]];
   
   (* Experiment Beginn - das bringt nix  *)
   If[OptionValue@Verbose,Print@"Now Starting Phases 2,3"];
    weightCond = True;
   If[ (OptionValue@LastStep == 1 ||  OptionValue@LastStep == 3), 
    ok3 = Check[
      sol3 = FindMinimum[{F[Table[ow[j], {j, 1, m}], 1, 1, 1] + 
                                  If[weightCond, n^OptionValue@WEexpo*Sum[(1 - ow[j]*EVM[j, j])^2, {j, 1, m}], 0],                                                               
                  Join[constr, {Sum[ow[j], {j, 1, m}] >  Apply[Plus, 1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]]},
                  Table[ow[j] > 0, {j, 1, m}] ]},
           Join[val0, Table[{ow[j], If[(EVM[j, j]/.sol) <minw,1/minw,1/EVM[j, j] /. sol]}, {j, 1, m}]]][[2]]; True, False];
    oww = Table[ow[j], {j, 1, m}] /. sol3; 
    If[ok3, sol = sol3;www=oww];
    oww = oww/Apply[Plus, oww];
    If[OptionValue@LastStep == 3, 
     ok3 = Check[sol3b = 
        Fsol[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol3], pfe0, pfl0, pfd0, Map[{First@#, Last@#} &, sol3]]; True, False];
     ];
    If[ok3, sol = sol3b;www=1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol3]] ];
   
   (* Experimene Ende *)
   (* Experiment 2 Beginn - das bringt nix  *)
   If[OptionValue@Verbose,Print@"Now Starting Last Phases"];
    If[ OptionValue@LastStep == 2, 
    weightCond = True;
    ok3 = 
     Check[sol3b = 
       FindMinimum[{F[Table[ow[j], {j, 1, m}], 1, 1, 1]/ Sqrt@Sum[(1/ow[j])*EVM[j, j], {j, 1, m}] +  
                    n^2*Sum[(1 - ow[j]*EVM[j, j])^2, {j, 1, m}],                                            
          Join[constr, {Sum[ow[j], {j, 1, m}] >  Apply[Plus, 1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol] ]},
           Table[ow[j] > minw, {j, 1, m}] ]},
         Join[val0, Table[{ow[j],    If[(EVM[j, j]/.sol) <minw,1/minw,1/EVM[j, j] /. sol]    }, {j, 1, m}]]][[2]]; True, False];
    oww = Table[ow[j], {j, 1, m}] /. sol3b; 
    oww = oww/Apply[Plus, oww]; 
    If[ok3, sol = sol3b; ok=ok3] ];
   (* Experiment 2 Ende *)

If[OptionValue@WeightVarBound>0,  
    Fsol[w_List, pfe_, pfl_, pfd_, val0_] :=  
       FindMinimum[{F[w, pfe, pfl, pfd]+OptionValue@WeightVarBound*Sum[(w[[j]]*EVM[j,j]-K0)^2,{j,1,m}], constr}, val0,
           MaxIterations->OptionValue@MaxIter (*,AccuracyGoal->prec,  PrecisionGoal->prec *)][[2]]
];
If[OptionValue@WeightVarBound<0,  
    Fsol[w_List, pfe_, pfl_, pfd_, val0_] :=  
       FindMinimum[{F[w, pfe, pfl, pfd], 
                    Join[constr,If[OptionValue@WeightVarBound<0,Table[w[[j]]*EVM[j,j]==K0,{j,1,m}],{}]]}, val0,
           MaxIterations->OptionValue@MaxIter (*,AccuracyGoal->prec,  PrecisionGoal->prec *)][[2]]
];



    If[ OptionValue@LastStep == 4,  (* Identity was makeMW, but seem snot a good idea *)
     ok4 = Check[sol4 = Fsol[If[ok,1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol],www], pfe0, pfl0, pfd0, 
       If[ok,Map[{First@#, Last@#} &, sol],val0 ] ]; True, False]; 
    If[ok4, sol = sol4; ok=True] ];
    If[ OptionValue@LastStep == 5, 
     ok5 = Check[sol5 = Fsol[Identity[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]], 0, 0, 0, Map[{First@#, Last@#} &, sol]]; True, False]; 
    If[ok5, sol = sol5; ok=True] ];
  If[ OptionValue@LastStep == 6, 
    If[OptionValue@Verbose,Print[{"LastStep6, weights",1/makeMW[Table[If[OptionValue@SqrtWeights6,Sqrt[EVM[k, k]],EVM[k, k]], {k, 1, m}] /. sol]}]];
     ok6 = Check[sol6 = Fsol[Identity[1/makeMW[Table[If[OptionValue@SqrtWeights6,Sqrt[EVM[k, k]],EVM[k, k]], {k, 1, m}] /. sol]], 
       OptionValue@StepP,OptionValue@StepP,OptionValue@StepP, Map[{First@#, Last@#} &, sol]]; True, False]; 
    If[ok6, sol = sol6; ok=True,If[!OptionValue@Quiet,Print["Warning: ok6 fail"]]] ];
 
    If[ OptionValue@LastStep == 7, 
     F7=Sum[wgt[k]*EVM[k, k], {k, 1, m}]+OptionValue@StepP*(FE + FL + FD)+Sum[(OptionValue@userOptim)[[j]]^2 ,{j,1,Length@OptionValue@userOptim}]+
       OptionValue@WPenalty*Length@data*Sum[(wgt[k]*EVM[k, k]-K0)^2,{k,1,m}];
     ok7 = Check[sol7 = FindMinimum[{F7, 
                                     Join[constr,{K0>0.001,1<Sum[wgt[k],{k,1,m}]},Map[(First@#>1/5*Last@#)&,Transpose@{Table[wgt[k],{k,1,m}],1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]} ],
                                       Map[(First@#<5*Last@#)&,Transpose@{Table[wgt[k],{k,1,m}],1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]} ] ]},  
                                    Join[{{K0,1}},Transpose@{Table[wgt[k],{k,1,m}],1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]},Map[{First@#, Last@#} &, sol ]] ]; True, False]; 
    If[ok7, sol = sol7[[2]]; ok=True] ];




If[OptionValue@doIterStep>0,

Module[{solpre,EEoff,fails=0,ErrOK1,ErrOK2,ww1,ww2,dec,sol1,sol2,sol0,F0,FS,infolist,lastsol,k,s,EO0,EO1,EO2,lastEO,WRstep=0.2,
            ww=normV[1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol]],ve0,datT=Transpose@data,GoF,varE,EEVL},
            EEVs=Map[Apply[Plus,Map[#^2&,NonNull@#]]&,AllEEs];
            varE[sol_]:= 1/n*EEVs/.sol;
            FS[w_List]:=Fsol[w/Mean[varE[sol]],  OptionValue@StepP,OptionValue@StepP,OptionValue@StepP, Map[{First@#, Last@#} &, sol]];
            ve0=Map[Function[{er}, er[[2]]  ],Errors];
            GoF[sol_]:=Module[{AE=AllEEs/.sol,LL=(lvv[#1] /. sol &) /@ latentvars},
            {   Mean@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Max@Abs@Flatten@Table[Table[Correlation0[AE[[j]], AE[[i]]], {i, j+1, m}], {j, 1, m-1}],
                Mean@Abs@Flatten@Table[Correlation0[LL[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@latentvars}],
                Mean@Abs@Flatten@Table[Correlation0[datT[[j]], AE[[i]]], {i, 1, m}, {j, 1, Length@observed}],
                Sum[ww[[j]]*1/n*Sum[If[NumberQ[AE[[j,i]]],AE[[j,i]]^2,0],{i,1,n}],{j,1,m}]}
             ] ;
            EEoff[sol_]:= {Length@eqs,1,1,1,1}.GoF[sol]; 
            sol=FS[ww]; EO0=EEoff[sol];
           If[OptionValue@Verbose, Print[{"Start Loop",OptionValue@watchParas /.sol }];
            Print[{"GF",GoF[sol]}]];
              Do[   lastEO=EEoff[sol];   fails=0;   Do[ 
                If[OptionValue@Verbose&&s==1,Print[{k, EEoff[sol],GoF[sol],OptionValue@watchParas/.sol,Round[NormV[ww],0.01]}]];
             ww1=NormV[ ReplacePart[ww,s->ww[[s]]*(1-WRstep) ]];ww2=NormV[ ReplacePart[ww,s->ww[[s]]*(1+WRstep) ]];
             ErrOK1 =  Check[solpre = FS[ww1];True, False];
             If[ErrOK1,sol1=solpre,sol1=sol];
             If[!ErrOK1,(*Print["ERR1"];*) GARGS=InsertWeights[arguments, ww1]; GWWW=ww1; ];
             ErrOK2 =  Check[solpre = FS[ww2];True, False];
             If[ErrOK2,sol2=solpre,sol2=sol];
             If[!ErrOK2,(*Print["ERR2"];*) GARGS=InsertWeights[arguments, ww2]; GWWW=ww2; ];
             EO1=EEoff[sol1];EO2=EEoff[sol2];
             sol0=sol;ww0=ww; dec=False;
             If[ErrOK1 && ErrOK2 && EO1<EO0 && EO1<EO2  , dec=True; sol=sol1;ww=ww1;EO0=EO1 ];
            If[ErrOK1 && ErrOK2 && EO2<EO0 && EO2<EO1 , dec=True; sol=sol2;ww=ww2;EO0=EO2 ];
            If[!dec,fails=fails+1]  ,  {s,1,Length[eqs]}  ];
              If[Abs[EO0-lastEO]/lastEO<OptionValue@Itereps,
                        If[OptionValue[Verbose],Print["Stationary point seems to be apporached "]]; 
                       res["iterEnd"]="Near stationary point";Break[];];
            If[fails==Length@eqs, If[OptionValue[Verbose],Print[{"too many non-improving dimensions, round ",k}]; 
            res["iterEnd"]="No incr dimension";Break[]]];
            If[OptionValue[Verbose],Print[{"Round",k,"EO=",EO0,"sol...=",Take[sol ,Min[3,Length[sol]]],"weights",Round[ww,0.001],"#fails",fails}]],
             {k, OptionValue[doIterStep]}];

] ];




   If[OptionValue@Verbose, Print[{"Starting report",If[ok,OptionValue@watchParas /. sol,"not ok"]}]];
   res = 
    Association["n" -> n, "data" -> data, "latentvars" -> latentvars, "Title"->OptionValue@Title,
     "eqs" -> eqs, "observed" -> observed, "ConvergenceOK"->ok,"val0"->val0,
     "paras" -> Complement[GetAllVariables[First /@ eqs], observed]];
   res["F"] = F1;
   res["constr"] = constr; 
   res["vars"] = GetAllVariables[Join[{F1}, constr]];
   res["sol"] = sol;
   
   If[!ok&&!OptionValue@Quiet,Print["Warning: slowSEM did not succeed"];Return[res]];

   res["weights"] = www;
   res["weightsN"] = res["weights"] /Apply[Plus,res["weights"]]; 
   res["weightsPost"]=1/makeMW[Table[EVM[k, k], {k, 1, m}] /. sol];
   res["weightsPostN"] = res["weightsPost"] /Apply[Plus,res["weightsPost"]]; 
   res["Fmin"] = F0 /. sol;
   res["Fmin0"] = F0 /. weight[_] :> 1/Sqrt[m] /. res["sol"];
   res["R"] = (F00 /. res["sol"])/(quoteData*n);
   res["R0"] = (F00 /. res["sol"])/(quoteData*n*m);
   res["parasol"] = Select[sol, MemberQ[res["paras"], First[#1]] &];
   res["LLs"] = (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLmeans"] = Mean /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["LLsds"] =  StandardDeviation /@ (lvv[#1] /. res["sol"] &) /@ latentvars;
   res["EEs"] = AllEEs /. sol;
   res["EVars"] = (Variance[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EEmeans"] = (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
   res["EMeans"] =  (Mean[Select[#1, NumberQ]] &) /@ res["EEs"];
    res["EE0s"] =  Table[Table[AllEEs[[j, i]], {i, 1, n}], {j, 1, Length[errvars]}] /.  sol;
   res["LEcors"] = Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], res["LLs"][[j, s]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
         {i, 1, m}, {j, 1, Length@latentvars}];
   res["DEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] && NumberQ[data[[s,j]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], data[[s,j]]}]], {s, 1, n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, Length@observed}];
   res["EEcors"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1,  n}]; Correlation0 @@ Transpose[dat]],
          {i, 1, m}, {j, 1, m}];
   res["EEcovs"] = 
    Table[Module[{dat = {}, s}, 
      Do[If[NumberQ[res["EE0s"][[i, s]]] &&  NumberQ[res["EE0s"][[j, s]]], 
        AppendTo[
         dat, {res["EE0s"][[i, s]], res["EE0s"][[j, s]]}]], {s, 1, 
        n}]; Covariance @@ Transpose[dat]], {i, 1, m}, {j, 1, m}];
    res["GOF"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Mean@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Mean@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
              (* "weightPostweightAgreement",Norm[res["weightsPost"]/Apply[Plus,res["weightsPost"]]-
                                                res["weights"]/Apply[Plus,res["weights"]] ], *)
                "SDWeightErr",StandardDeviation[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]};     
  res["GOF1"]=Check[Mean[{m,1,1,1,0}*Select[res["GOF"],NumberQ]],Null];
  If[GOF1i*1.5<res["GOF1"],res["ConvergenceOK"]=False;If[!OptionValue@Quiet,Print[{"GOF1 convergence warning",GOF1i,res["GOF1"],{b1,b2,b3}/.res["sol"]}]]];
(*!!!*)    res["GOF1"]=Mean[GOFList/.res["sol"]];
res["GOFL"]=Map[Mean,GOFList2/.res["sol"]];

   res["GOFM"]={"ResVar",1/m*Sqrt@Sum[res["EEcovs"][[j,j]],{j,1,m}],"EECorMean",Max@Abs@Flatten[Table[Table[res["EEcors"][[i,j]],{i,j+1,m}],{j,1,m-1}]],
               "LECorMean",Max@Abs@Flatten@res["LEcors"],"DECorMean",Mean@Abs@Flatten@res["DEcors"],
               "SDWeightErr",StandardDeviation[(res["weights"]/Apply[Plus,res["weights"]])*
                                   (Table[res["EEcovs"][[j,j]],{j,1,m}])/Apply[Plus,Table[res["EEcovs"][[j,j]],{j,1,m}]] ]};     
  res["GOF2"]=Check[Mean[{m,1,1,1,0}*Select[res["GOFM"],NumberQ]],Null];


 res["NormedParas"]=calcNormedParas[res];

   res["eqssol"] = 
    Table[{eqs[[i, 1]] /. res["parasol"], res["EVars"][[i]]}, {i, 1, Length@eqs}];
   res["eqssolNorm"] = 
    Table[{eqs[[i, 1]] /. res["NormedParas"], res["EVars"][[i]]}, {i, 1, Length@eqs}]/.res["sol"];

  If[OptionValue@calcDataFit>=0, 
      res["dataFit"]=CalcDataFit[res,OptionValue@calcDataFit>0,OptionValue@calcDataFit],
      res["dataFit"]="Data fit not calculated" ] ;


    res["time"]= AbsoluteTime[]-t0;
    If[OptionValue@Quiet,On[FindMinimum::lstol];On[FindMinimum::bddir];On[FindMinimum::eit];On[FindMinimum::sdir];
                 On[FindMinimum::cvmit];On[FindMinimum::acceptlev];On[FindMinimum::lstol];On[FindMinimum::cvmit];
                 On[FindMinimum::sszero];On[FindMinimum::infeas];On[FindMinimum::sszero];];

   res];



